(* ::Package:: *)

(*:Package Version: 3.02 *)

(*:Version History: 1.01 (2011.11.11 - 2012.01.30)
                    1.02 (2012.01.31 - 2012.04.21)
                    1.03 (2012.04.22 - 2012.12.16)
                    1.04 (2012.12.17 - 2013.03.15)
                    1.05 (2013.03.16 - 2013.09.30)
                    1.06 (2013.10.01 - 2014.05.06):ContoursThickness
                    1.07 (2014.05.07 - 2015.02.18):origMeshF,origMeshInvF (LogShift1)
                    1.08 (2015.02.19 - 2015.09.23):GridLinesPlot, RePlot3D, ImPlot3D
                    1.09 (2015.09.24 - 2016.02.13):origColorScheme, PlotRegion
                    1.10 (2016.02.14 - 2016.09.21):MeshFunctions
                    1.11 (2016.09.22 - 2017.11.26):RiemannSphere
                    1.12 (2017.11.27 - 2018.11.08):If->Min
                    1.13 (2018.11.09 - 2018.11.16):ClippingStyle (Re,ImPlot3D)
                    1.14 (2018.11.17 - 2019.06.28):AbsArgLogPlot3D, AbsArgArcSinhPlot3D
                    1.15 (2019.06.29 - 2019.07.04):New color scheme
                    2.01 (2019.07.05 - 2020.02.26):Mesh->None(False), DefinitionDomain
                    2.02 (2020.02.27 - 2020.09.05):ClippingStyle (Re,ImPlot3D)
                    2.03 (2020.09.06 - 2021.04.12):New color scheme
                    3.01 (2021.04.13 - 2021.07.09):MeshFunctions
                    3.02 (2021.07.10 - 20xx.xx.xx):New color scheme
 *)

(*:Compatible Mathematica Version: 8.0 *)

(*:Copyright: Copyright 2011.11-2021.07, Souichiro Ikebe *)

(*:Author: Souichiro Ikebe *)

(*:Name: Graphics`MathematicalFunctionPlot` *)

(*:Title: MathematicalFunctionPlot *)

(*:Keywords: 
  GridLinesPlot, GridLinesParametricPlot, AbsArgPlot, ReImPlot, AbsArgPlot3D,
  ArgDensityPlot, AbsArgLogPlot3D, AbsArgArcSinhPlot3D, FundamentalRegionPlot, 
  FundamentalRegionPlot3D, RePlot3D, ImPlot3D, TopographyPlot3D, 
  RiemannSpherePlot3D, RiemannSphereDensityPlot, RiemannSphereLogPlot3D, 
  RiemannSphereArcSinhPlot3D, NonMeshAbsArgPlot3D, NonMeshArgDensityPlot*)

(*:Sources: 

  Roman Maeder, Programming in Mathematica (Second Edition), 
  Addison-Wesley Publishing Company,Inc. (1991).

  Theodore W. Gray & Jerry Glynn, Exploring mathematics with Mathematica, 
  Addison-Wesley Publishing Company,Inc. (1991).

  Sal Mangano, Mathematica Cookbook, 
  O'Reilly Media,Inc. (2011).

*)

(********************************************************************************)




BeginPackage["Graphics`MathematicalFunctionPlot`"]




Unprotect[
  GridLinesPlot, GridLinesParametricPlot, GridLinesColor, FundamentalRectangle, 
  AbsArgPlot, ArgColorScheme, ReImPlot, ReImFillingColors, ZeroPointsStyle, 
  AbsArgPlot3D, ArgDensityPlot, AbsContours, ArgContours, ContoursColor, 
  ContoursThickness, ContoursColorAbsMax, SurfaceProperty, DefinitionDomain, 
  AbsArgLogPlot3D, AbsArgArcSinhPlot3D, TicksExpStep, 
  FundamentalRegionPlot, FundamentalRegionPlot3D, RegionColor, BranchCutColor, 
  RePlot3D, ImPlot3D, ReContours, ImContours, ExclusionsType, ClippingPlane, 
  TopographyPlot3D, LandColorScheme, LandFillingColor, SeaLevel, SeaColor, SeaWave, 
  RiemannSpherePlot3D, RiemannSphereDensityPlot, RiemannSphereLogPlot3D, 
  RiemannSphereArcSinhPlot3D, ZeroPointAltitude, FunctionMaxRange, RiemannSphereType, 
  AdjointSphere, AdjointMesh, AdjointMeshOpacity, AdjointSphereOpacity, 
  AdjointSphereRadius, AdjointArrowLength, 
  NonMeshAbsArgPlot3D, NonMeshArgDensityPlot, ContoursOpacity]



GridLinesPlot::usage = 
"GridLinesPlot[f[x],{x, xmin, xmax},opts] plots the image of the two
 dimension plot of the function f with grid lines."

GridLinesParametricPlot::usage = 
"GridLinesParametricPlot[{f1[t], f2[t]},{t, tmin, tmax},opts] plots the
 image of the two dimension parametric plot of the function f1 and
 f2 with grid lines."

GridLinesColor::usage = 
"GridLinesColor -> {xcolor,ycolor} is an option of the GridLinesPlot or 
 GridLinesParametricPlot in designate the color of lines to draw.
 This option Default is GridLinesColor -> {Hue[0.5],Hue[0.5]}."

FundamentalRectangle::usage = 
"FundamentalRectangle -> {{x0,y0},{x1,y1}} is an option of the
 GridLinesPlot or GridLinesParametricPlot in designate the fundamental 
 rectangle of grid lines paper.
 This option Default is FundamentalRectangle -> {{0,0},{1,1}}."

AbsArgPlot::usage = 
"AbsArgPlot[f[x],{x, xmin, xmax},opts] plots the image of the two
 dimension plot of the function Abs(f) with grid lines and area 
 between the curve and the var axis filled."

ReImPlot::usage = 
"ReImPlot[f[x],{x, xmin, xmax},opts] plots the image of the two
 dimension plot of the function {Re(f),Im(f)} with grid lines 
 and area between the curve and the var axis filled."

ArgColorScheme::usage = 
"ArgColorScheme -> {colSchStr, {satu, brig, opac}} is an option of the
 AbsArgPlot, AbsArgPlot3D, ArgDensityPlot, AbsArgLogPlot3D, AbsArgArcSinhPlot3D, 
 RiemannSpherePlot3D, RiemannSphereDensityPlot, RiemannSphereLogPlot3D, 
 RiemannSphereArcSinhPlot3D, NonMeshAbsArgPlot3D or NonMeshArgDensityPlot 
 in designate the colors of argument scheme.
 colSchStr is string of color scheme name, satu is saturation, 
 brig is brightness, and opac is opacity. 
 This option Default is ArgColorScheme -> {Hue,{1,1,0.6}}(AbsArgPlot),
 ArgColorScheme -> {Hue,{1,1,1}}(AbsArgPlot3D, ArgDensityPlot, AbsArgLogPlot3D, 
 AbsArgArcSinhPlot3D, RiemannSpherePlot3D, RiemannSphereDensityPlot, 
 RiemannSphereLogPlot3D, RiemannSphereArcSinhPlot3D, NonMeshAbsArgPlot3D, 
 NonMeshArgDensityPlot)."

ReImFillingColors::usage = 
"ReImFillingColors -> {{recol1,recol2},{imcol1,imcol2}} is an option
 of the ReImPlot in designate the colors of filling area. 
 recol1(or recol2) is area of positive(or negative) real values and 
 imcol1(or imcol2) is area of positive(or negative) imaginal values.
 This option Default is ReImFillingColors -> 
 {{Hue[0,1,1,0.6],Hue[0.5,1,1,0.6]},{Hue[0.25,1,1,0.6],Hue[0.75,1,1,0.6]}}."

ZeroPointsStyle::usage = 
"ZeroPointsStyle -> {style1,style2,...} is an option of the ReImPlot
 in designate the graphics primitive styles on the point of zeros. 
 This option Default is ZeroPointsStyle -> 
 {PointSize[Large],Hue[0],Opacity[0.5]}."

AbsArgPlot3D::usage = 
"AbsArgPlot3D[f[z],{z, z0, z1},opts] plots the image of the three
 dimension plot of the function Abs(f) on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and 
 right-over) of rectangle on the complex plane.
 It's Surface determination coloring scheme by values of Arg(f)."

ArgDensityPlot::usage = 
"ArgDensityPlot[f[z],{z, z0, z1},opts] plots the image of the density
 plot of the function Arg(f) on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and 
 right-over) of rectangle on the complex plane.
 ArgDensityPlot use the same coloring in AbsArgPlot3D."

AbsArgLogPlot3D::usage = 
"AbsArgLogPlot3D[f[z],{z, z0, z1},opts] plots the image of the three
 dimension plot of the function Log[10,Abs(f)] on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and 
 right-over) of rectangle on the complex plane.
 It's Surface determination coloring scheme by values of Arg(f)."

AbsArgArcSinhPlot3D::usage = 
"AbsArgArcSinhPlot3D[f[z],{z, z0, z1},opts] plots the image of the three
 dimension plot of the function ArcSinh[Abs(f)] on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and 
 right-over) of rectangle on the complex plane.
 It's Surface determination coloring scheme by values of Arg(f)."

AbsContours::usage = 
"AbsContours -> {stepFuncStr,logBase,contMin} is an option of the AbsArgPlot3D,
 ArgDensityPlot, AbsArgLogPlot3D, AbsArgArcSinhPlot3D, FundamentalRegionPlot, 
 FundamentalRegionPlot3D, TopographyPlot3D, RiemannSpherePlot3D, 
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D, RiemannSphereArcSinhPlot3D, 
 NonMeshAbsArgPlot3D or NonMeshArgDensityPlot determine the contours of Abs(f). 
 stepFuncStr is functions name determine sampling values of contours, 
 logBase is values determine base of Logarithm use in stepFuncStr functions,
 contMin is values determine minimum altitude contours.
 This option Default is AbsContours -> {Log,2,-5}(AbsArgPlot3D, 
 FundamentalRegionPlot3D, AbsArgArcSinhPlot3D, RiemannSpherePlot3D, 
 RiemannSphereLogPlot3D, RiemannSphereArcSinhPlot3D), 
 AbsContours -> {Log,2,{-5,30}}(ArgDensityPlot, FundamentalRegionPlot, 
 RiemannSphereDensityPlot),
 AbsContours -> {Log,2}(AbsArgLogPlot3D, NonMeshAbsArgPlot3D, 
 NonMeshArgDensityPlot), 
 AbsContours -> {LinearLog,2,-10}(TopographyPlot3D)."

ArgContours::usage = 
"ArgContours -> c1 is an option of the AbsArgPlot3D, ArgDensityPlot, 
 AbsArgLogPlot3D, AbsArgArcSinhPlot3D, FundamentalRegionPlot, 
 FundamentalRegionPlot3D, TopographyPlot3D, RiemannSpherePlot3D, 
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D 
 determine the contours of Arg(f).
 c1 is integer values determine division of argument Arg(f).
 This option Default is ArgContours -> 8."

ContoursColor::usage = 
"ContoursColor -> {{abscol1,abscol2,...},argcol} is an option of the AbsArgPlot3D,
 ArgDensityPlot, AbsArgLogPlot3D, AbsArgArcSinhPlot3D, FundamentalRegionPlot, 
 FundamentalRegionPlot3D, TopographyPlot3D, RiemannSpherePlot3D, 
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D 
 in designate the colors on contours.
 {abscol1,abscol2,...} is list of blending colors on contours of Abs(f),
 argcol is colors on contours of Arg(f). 
 This option Default is ContoursColor -> 
 {{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}(AbsArgPlot3D, 
 AbsArgLogPlot3D, AbsArgArcSinhPlot3D, FundamentalRegionPlot3D, RiemannSpherePlot3D, 
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D, RiemannSphereArcSinhPlot3D),
 ContoursColor -> {{GrayLevel[0,0.8],GrayLevel[1,0.8]},GrayLevel[0,0.4]}
 (ArgDensityPlot, FundamentalRegionPlot).
 This option usage ContoursColor -> {{colP1,colN1},{colP2,colN2}} at the 
 RePlot3D or ImPlot3D in designate the colors on contours. 
 colP1(colN1) is colors on contours of positive(negative) Re(f) or Im(f). 
 colP2(colN2) is colors on contours of positive(negative) Im(f) or Re(f). 
 Here option Default is ContoursColor -> {{Hue[0,1,0.8,1],Hue[0.5,1,0.8,1]},\[IndentingNewLine] {Hue[0.25,1,0.8,0.5],Hue[0.75,1,0.8,0.5]}}(RePlot3D),
 ContoursColor -> {{Hue[0.25,1,0.8,1],Hue[0.75,1,0.8,1]},\[IndentingNewLine] {Hue[0,1,0.8,0.5],Hue[0.5,1,0.8,0.5]}}(ImPlot3D),
 ContoursColor -> {GrayLevel[0,0.4],GrayLevel[0,0.4]}(TopographyPlot3D)."

ContoursThickness::usage = 
"ContoursThickness -> {absV, argV} is an option of the AbsArgPlot3D, 
 ArgDensityPlot, AbsArgLogPlot3D, AbsArgArcSinhPlot3D, FundamentalRegionPlot, 
 FundamentalRegionPlot3D, TopographyPlot3D, RiemannSpherePlot3D, 
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D 
 determine the thickness of the contour of Abs(f) and Arg(f). 
 In the Plot3D-system, default of this option is ContoursThickness -> 
 {Thickness[0.0015],Thickness[0.0015]}.
 In the DensityPlot-system, default of this option is ContoursThickness -> 
 {Thickness[0.001],Thickness[0.001]}.
 ContoursThickness -> {reV, imV} (or {imV, reV}) is an option of the RePlot3D 
 (or ImPlot3D) determine the thickness of the contour of Re(f) and Im(f) 
 (or Im(f) and Re(f)). 
 In the RePlot3D or ImPlot3D, default of this option is ContoursThickness -> 
 {Thickness[0.0015],Thickness[0.0015]}."

ContoursColorAbsMax::usage = 
"ContoursColorAbsMax -> c1 is an option of the ArgDensityPlot,
 FundamentalRegionPlot, RiemannSphereDensityPlot or NonMeshArgDensityPlot 
 determine the contours value of Abs(f) apply maximum color of ContoursColor[[1]]. 
 This option Default is ContoursColorAbsMax -> 20.
 When the ContoursColorAbsMax -> {c0,c1}, c0 that is the minimum color of 
 ContoursColor[[1]] also given."

SurfaceProperty::usage = 
"SurfaceProperty -> g1 is an option of the AbsArgPlot3D, AbsArgLogPlot3D,
 AbsArgArcSinhPlot3D, FundamentalRegionPlot3D, RePlot3D, ImPlot3D, 
 TopographyPlot3D, RiemannSpherePlot3D, RiemannSphereDensityPlot, 
 RiemannSphereLogPlot3D, RiemannSphereArcSinhPlot3D or NonMeshAbsArgPlot3D 
 determine the color property on the surface. 
 This option Default is SurfaceProperty -> Specularity[GrayLevel[0.5],40]."

DefinitionDomain::usage = 
"DefinitionDomain -> (ineq) is an option of the AbsArgPlot3D, ArgDensityPlot, 
 AbsArgLogPlot3D, AbsArgArcSinhPlot3D, FundamentalRegionPlot3D, 
 FundamentalRegionPlot, RePlot3D, ImPlot3D, NonMeshAbsArgPlot3D or NonMeshArgDensityPlot 
 determine inequalities of the domain of definition."

TicksExpStep::usage = 
"TicksExpStep -> n1 is an option of the AbsArgLogPlot3D or AbsArgArcSinhPlot3D
 determine the exponential step of the ticks value.
 This option Default is TicksExpStep -> 1."

FundamentalRegionPlot::usage = 
"FundamentalRegionPlot[f[z],{z, z0, z1},opts] plots the image of the density
 plot of the fundamental regions on the complex plane. 
 f is complex valued function, z0 and z1 is vertex(left-under and right-over) 
 of rectangle on the complex plane."

FundamentalRegionPlot3D::usage = 
"FundamentalRegionPlot3D[f[z],{z, z0, z1},opts] plots the image of the three
 dimension plot of the function Abs(f) on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and 
 right-over) of rectangle on the complex plane.
 It's Surface coloring use the same coloring in FundamentalRegionPlot."

RegionColor::usage = 
"RegionColor -> {{pcol1,pcol2,...},{ncol1,ncol2,...}} is an option of the
 FundamentalRegionPlot, FundamentalRegionPlot3D, RePlot3D or ImPlot3D 
 determine the colors on fundamental regions.
 {pcol1,...} is image gradient colors on positive argument(or Re(f) or Im(f)) 
 regions, 
 {ncol1,...} is image gradient colors on negative argument(or Re(f) or Im(f)) 
 regions.
 This option Default is RegionColor -> {{Hue[0],Hue[0.1]},{Hue[0.5],Hue[0.6]}}
 (FundamentalRegionPlot or FundamentalRegionPlot3D),
 RegionColor -> {{Hue[0],Hue[0,1,1,0.4]},{Hue[0.5],Hue[0.5,1,1,0.4]}}(RePlot3D),
 RegionColor -> {{Hue[0.25],Hue[0.25,1,1,0.4]},{Hue[0.75],Hue[0.75,1,1,0.4]}}
 (ImPlot3D)."

BranchCutColor::usage = 
"BranchCutColor -> col1 is an option of the FundamentalRegionPlot,
 FundamentalRegionPlot3D, RePlot3D or ImPlot3D determine the colors on 
 branch cut lines.
 This option Default is BranchCutColor -> GrayLevel[0]."

ExclusionsType::usage = 
"ExclusionsType -> typ1 is an option of the FundamentalRegionPlot,
 FundamentalRegionPlot3D, RePlot3D or ImPlot3D determine the type or equation 
 and inequality of branch cut lines. 
 Type of typ1 is Automatic, None, True. Equation and inequality of typ1 
 is examples {equ[z]==0}, {{equ[z]==0,ine[z]<0}}.
 This option Default is ExclusionsType -> Automatic."

RePlot3D::usage = 
"RePlot3D[f[z],{z, z0, z1},opts] plots the image of the three dimension plot
 of the function Re(f) on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and right-over) 
 of rectangle on the complex plane.
 It's Surface determination coloring by positive or negative values of Re(f)."

ImPlot3D::usage = 
"ImPlot3D[f[z],{z, z0, z1},opts] plots the image of the three dimension plot
 of the function Im(f) on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and right-over) 
 of rectangle on the complex plane.
 It's Surface determination coloring by positive or negative values of Im(f)."

ReContours::usage = 
"ReContours -> {stepFuncStr,{initC,logBase}} or ReContours -> 
 {stepFuncStr,{initC,logBase},maxIndex} is an option of the RePlot3D or ImPlot3D 
 determine the contours of Re(f). 
 stepFuncStr is functions name determine sampling values of contours,
 initC is initial value at i=1, here i was index of contours table,
 logBase is values determine base of Logarithm use in stepFuncStr functions,
 maxIndex is value determine index(i) of maximum altitude contours.
 This option Default in the RePlot3D is ReContours -> {Linear,{1,1}},
 In the ImPlot3D is ReContours -> {Log,{2,2},Floor[ASF[Log,{2,2},5*pMax]]}.
 Here ASF is inverse function of sampling stepFuncStr functions, and pMax is maximum
 either two PlotRange option values of absolute."

ImContours::usage = 
"ImContours -> {stepFuncStr,{initC,logBase}} or ReContours -> 
 {stepFuncStr,{initC,logBase},maxIndex} is an option of the RePlot3D or ImPlot3D 
 determine the contours of Im(f). 
 stepFuncStr is functions name determine sampling values of contours,
 initC is initial value at i=1, here i was index of contours table,
 logBase is values determine base of Logarithm use in stepFuncStr functions,
 maxIndex is value determine index(i) of maximum altitude contours.
 This option Default in the ImPlot3D is ImContours -> {Linear,{1,1}},
 In the RePlot3D is ImContours -> {Log,{2,2},Floor[ASF[Log,{2,2},5*pMax]]}.
 Here ASF is inverse function of sampling stepFuncStr functions, and pMax is maximum
 either two PlotRange option values of absolute."

ClippingPlane::usage = 
"ClippingPlane -> {styMin,styMax} is an option of the RePlot3D or ImPlot3D
 determine the planes style of maximum and minimum surface clipping.
 ClippingPlane -> True determine same {styMin,styMax}={RegionColor[[2,2]],
 RegionColor[[1,2]]}.
 This option Default is ClippingPlane -> None."

TopographyPlot3D::usage = 
"TopographyPlot3D[f[z],{z, z0, z1},opts] plots the image of the three dimension 
 plot of the function Abs(f) on the complex plane with sea level plane.
 f is complex valued function, z0 and z1 is vertex(left-under and 
 right-over) of rectangle on the complex plane.
 It's Surface determination coloring original scheme by values of Abg(f)."

LandColorScheme::usage = 
"LandColorScheme -> {colSchStr, {satu, brig, opac}} is an option of the
 TopographyPlot3D in designate the colors of absolute scheme.
 colSchStr is string of color scheme name, satu is saturation, 
 brig is brightness, and opac is opacity. 
 This option Default is LandColorScheme -> {geography,{1,1,1}}."

LandFillingColor::usage = 
"LandFillingColor -> color1 is an option
 of the TopographyPlot3D in designate the colors of landy bottom filling area. 
 This option Default is LandFillingColor -> Hue[0.1,0.5,0.4]."

SeaLevel::usage = 
"SeaLevel -> num is an option of the TopographyPlot3D in designate the sea
 surface of level value. 
 This option Default is SeaLevel -> 1."

SeaColor::usage = 
"SeaColor -> {surfaceCol,bottomCol} is an option of the
 TopographyPlot3D in designate the colors of sea water.
 surfaceCol is color of sea surface,
 bottomCol is color of sea bottom filling area. 
 This option Default is SeaColor -> {Hue[0.6,0.8,1,0.5],Hue[0.6,1,0.8,0.6]}."

SeaWave::usage = 
"SeaWave -> {waveAmp, {xPeriod, yPeriod}} is an option of the TopographyPlot3D in 
 waves factor of sea.
 waveAmp is amplitude of sea waves, xPeriod is x-direction period of sea waves and 
 yPeriod is y-direction period of sea waves. 
 This option Default is SeaWave -> {0,{0,0}}."

RiemannSpherePlot3D::usage = 
"RiemannSpherePlot3D[f[z],{z,r,{phi0,phi1}},opts] plots the image of the
 three dimension plot of the function Abs(f) on the Riemann-sphere.
 f is complex valued function, r is a radius of ploting region in circle 
 on the complex plane, {phi0,phi1} is mininum and maximum values of the 
 azimuth angle.
 It's Surface determination coloring scheme by values of Arg(f)."

RiemannSphereDensityPlot::usage = 
"RiemannSphereDensityPlot[f[z],{z,r,{phi0,phi1}},opts] plots the image of
 the density plot of the function Arg(f) on the Riemann-sphere.
 f is complex valued function, r is a radius of ploting region in circle 
 on the complex plane, {phi0,phi1} is mininum and maximum values of the 
 azimuth angle.
 RiemannSphereDensityPlot use the same coloring in RiemannSpherePlot3D."

RiemannSphereLogPlot3D::usage = 
"RiemannSphereLogPlot3D[f[z],{z,r,{phi0,phi1}},opts] plots the image of
 the three dimension plot of the function Log[10,Abs(f)] on the Riemann-
 sphere.
 f is complex valued function, r is a radius of ploting region in circle 
 on the complex plane, {phi0,phi1} is mininum and maximum values of the 
 azimuth angle.
 It's Surface determination coloring scheme by values of Arg(f)."

RiemannSphereArcSinhPlot3D::usage = 
"RiemannSphereArcSinhPlot3D[f[z],{z,r,{phi0,phi1}},opts] plots the image
 of the three dimension plot of the function ArcSinh[Abs(f)] on the 
 Riemann-sphere.
 f is complex valued function, r is a radius of ploting region in circle 
 on the complex plane, {phi0,phi1} is mininum and maximum values of the 
 azimuth angle.
 It's Surface determination coloring scheme by values of Arg(f)."

ZeroPointAltitude::usage = 
"ZeroPointAltitude -> v1 is an option of the RiemannSpherePlot3D,
 RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D.
 v1 is value of the level of zero point on the Riemann-sphere.
 This option Default is ZeroPointAltitude -> 10."

FunctionMaxRange::usage = 
"FunctionMaxRange -> v1 is an option of the RiemannSpherePlot3D,
 RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D.
 v1 is value of the maximum ploting range on the Riemann-sphere.
 This option Default is FunctionMaxRange -> 10(RiemannSpherePlot3D),
 FunctionMaxRange -> 10000(RiemannSphereLogPlot3D, RiemannSphereArcSinhPlot3D)."

RiemannSphereType::usage = 
"RiemannSphereType -> cnst is an option of the RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D.
 cnst=1 is type of infinity at the upper point on the Riemann-sphere, 
 or cnst=2 is type of infinity at the lower point on the Riemann-sphere.
 This option Default is RiemannSphereType -> 1."

AdjointSphere::usage = 
"AdjointSphere -> True or False is an option of the RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D.
 This option determines whether to draw the adjoint Riemann-sphere.
 This option Default is AdjointSphere -> True."

AdjointMesh::usage = 
"AdjointMesh -> {absMeshV,argMeshV} is an option of the RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D 
 in designate the half values of mesh lines on the adjoint Riemann-sphere.
 absMeshV is mesh of absolute values, and argMeshV is mesh of argument.
 This option Default is AdjointMesh -> {6,4}."

AdjointMeshOpacity::usage = 
"AdjointMeshOpacity -> o1 is an option of the RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D 
 in designate the values of mesh colors opacity on the adjoint Riemann-sphere.
 o1 is the real values in interval 0 and 1.
 This option Default is AdjointMeshOpacity -> 1(RiemannSpherePlot3D, 
 RiemannSphereLogPlot3D, RiemannSphereArcSinhPlot3D),
 AdjointMeshOpacity -> 0.7(RiemannSphereDensityPlot)."

AdjointSphereRadius::usage = 
"AdjointSphereRadius -> r1 is an option of the RiemannSphereDensityPlot.
 r1 is value of the radius with the adjoint Riemann-sphere.
 This option Default is AdjointSphereRadius -> 1.2."

AdjointSphereOpacity::usage = 
"AdjointSphereOpacity -> o1 is an option of the RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D 
 in designate the values of surface colors opacity on the adjoint Riemann-sphere.
 o1 is the real values in interval 0 and 1.
 This option Default is AdjointSphereOpacity -> 0.1."

AdjointArrowLength::usage = 
"AdjointArrowLength -> {len0,lenInf} is an option of the RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D or RiemannSphereArcSinhPlot3D 
 in designate the values of length of arrows on the adjoint Riemann-sphere.
 {len0,lenInf} is the real positive values list. len0 is length of arrow of zero 
 point, and lenInf is length of arrow of infinity point on the Riemann-sphere. 
 This option Default is AdjointArrowLength -> {0.3,0.3}."

NonMeshAbsArgPlot3D::usage = 
"NonMeshAbsArgPlot3D[f[z],{z, z0, z1},opts] plots the image of the three
 dimension plot of the function Abs(f) on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and 
 right-over) of rectangle on the complex plane.
 It's Surface determination coloring scheme by values of Arg(f) and Abs(f)."

NonMeshArgDensityPlot::usage = 
"NonMeshArgDensityPlot[f[z],{z, z0, z1},opts] plots the image of the density
 plot of the function Arg(f) and Abs(f) on the complex plane.
 f is complex valued function, z0 and z1 is vertex(left-under and 
 right-over) of rectangle on the complex plane.
 ArgDensityPlot use the same coloring in AbsArgPlot3D."

ContoursOpacity::usage = 
"ContoursOpacity -> o1 is an option of the NonMeshAbsArgPlot3D or
 NonMeshArgDensityPlot in designate the values of contours interval colors 
 opacity. o1 is the real values in interval 0 and 1.
 This option Default is ContoursOpacity -> 0.4."



Begin["`Private`"]



(******************************* Common Code *******************************)

(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*   GridLinesPlot, GridLinesParametricPlot,   *)
(*   AbsArgPlot, ReImPlot                      *)
(***********************************************)

gridltable[{x0_,x1_},{xmin_,xmax_},gridCol_]:=
  Module[{dx=(x1-x0)},
  Table[{x0+i*dx,{Directive[gridCol,Opacity[opRatio[i]]]}},
     {i,IntegerPart[(xmin-x0)/dx-1],IntegerPart[(xmax-x0)/dx+1],1/10}]];

opRatio[i_]:=
  If[Mod[2 i,2]===0,1,If[Mod[2 i,2]===1,0.5,0.25]];



(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*  AbsArgPlot, AbsArgPlot3D, ArgDensityPlot,  *) 
(*  AbsArgLogPlot3D, AbsArgArcSinhPlot3D,      *) 
(*  RiemannSpherePlot3D,                       *) 
(*  RiemannSphereDensityPlot,                  *) 
(*  RiemannSphereLogPlot3D,                    *) 
(*  RiemannSphereArcSinhPlot3D,                *) 
(*  NonMeshAbsArgPlot3D,                       *)
(*  NonMeshArgDensityPlot                      *)
(***********************************************)

argColSchOptCases[opv_]:=
  If[Length[opv]===0 && Head[opv]===String,{opv,{1,1,1}},
  If[Length[opv]===1,Join[opv,{{1,1,1}}],
  If[Length[opv]===2 && Length[opv[[2]]]===2,Insert[opv,1,{2,3}],
     opv]]];

origColorScheme[colName_String,par_,{sat_,bri_,opac_}]:=
  Which[
    colName==="Hue",
      Hue[par,sat,bri,opac],
    colName==="DayAndNight",
      Blend[{GrayLevel[1-sat,opac],Hue[0.75,sat,bri,opac],Hue[0.5,0.4*sat,bri,opac],
        Hue[0.3,0.2*sat,bri,opac],Hue[0.15,0.4*sat,bri,opac],Hue[0.05,sat,bri,opac],
        GrayLevel[1-sat,opac]},Mod[par+0.5,1]],
    colName==="Weather",
      Blend[{GrayLevel[0.9*bri,opac],Hue[0.15,sat,bri,opac],
        Hue[0.1,0.8*sat,bri,opac],GrayLevel[0.5*bri,opac],
        Hue[0.6,0.8*sat,0.8*bri,opac],Hue[0.55,sat,bri,opac],
        GrayLevel[0.9*bri,opac]},Mod[par,1]],
    colName==="NeonSign",
      Blend[{Hue[0,sat,bri,opac],GrayLevel[1-sat,opac],Hue[1/12,sat,bri,opac],
        GrayLevel[1-sat,opac],Hue[2/12,sat,bri,opac],GrayLevel[1-sat,opac],
        Hue[3/12,sat,bri,opac],GrayLevel[1-sat,opac],Hue[4/12,sat,bri,opac],
        GrayLevel[1-sat,opac],Hue[5/12,sat,bri,opac],GrayLevel[1-sat,opac],
        Hue[6/12,sat,bri,opac],GrayLevel[1-sat,opac],Hue[7/12,sat,bri,opac],
        GrayLevel[1-sat,opac],Hue[8/12,sat,bri,opac],GrayLevel[1-sat,opac],
        Hue[9/12,sat,bri,opac],GrayLevel[1-sat,opac],Hue[10/12,sat,bri,opac],
        GrayLevel[1-sat,opac],Hue[11/12,sat,bri,opac],GrayLevel[1-sat,opac],
        Hue[0,sat,bri,opac]},Mod[par,1]],
    colName==="RedAndCyan",
      If[0<=Mod[par,1]<0.5,
       Blend[{Hue[0,sat,bri,opac],Hue[0.1,sat,bri,opac]},Mod[2*par,1]],
       Blend[{Hue[0.5,sat,bri,opac],Hue[0.6,sat,bri,opac]},Mod[2*par,1]]],
    colName==="YellowAndPurple",
      If[0<=Mod[par,1]<0.5,
       Blend[{Hue[0.13,sat,bri,opac],Hue[0.18,sat,bri,opac]},Mod[2*par,1]],
       Blend[{Hue[0.73,sat,bri,opac],Hue[0.78,sat,bri,opac]},Mod[2*par,1]]],
    colName==="ComplexArgument4",
      Which[0<=Mod[par,1]<1/4,Hue[1/8,sat,bri,opac],
        1/4<=Mod[par,1]<1/2,Hue[3/8,sat,bri,opac],
        1/2<=Mod[par,1]<3/4,Hue[5/8,sat,bri,opac],
        3/4<=Mod[par,1]<1,Hue[7/8,sat,bri,opac]],
    colName==="MicrosoftWindows4",
      Which[0<=Mod[par,1]<1/4,Hue[2/5,sat,0.9*bri,opac],
        1/4<=Mod[par,1]<=1/2,Hue[59/60,0.8*sat,bri,opac],
        1/2<=Mod[par,1]<3/4,Hue[7/12,sat,bri,opac],
        3/4<=Mod[par,1]<1,Hue[7/48,0.9*sat,bri,opac]],
    colName==="NeonGenesisEvangelion4",
      Which[0<=Mod[par,1]<1/4,Hue[1/36,sat,0.9*bri,opac],
        1/4<=Mod[par,1]<=1/2,Hue[1/4,sat,bri,opac],
        1/2<=Mod[par,1]<3/4,Hue[37/48,sat,bri,opac],
        3/4<=Mod[par,1]<1,Hue[7/12,0.35*sat,bri,opac]],
    colName==="NIST4",
      Which[0<=Mod[par,1]<1/4,Hue[2/3,sat,bri,opac],
        1/4<=Mod[par,1]<=1/2,Hue[1/3,sat,bri,opac],
        1/2<=Mod[par,1]<3/4,Hue[0,sat,bri,opac],
        3/4<=Mod[par,1]<1,Hue[1/6,sat,bri,opac]],
    colName==="Sunset",
      Blend[{GrayLevel[1-sat,opac],Hue[0,sat,bri,opac],Hue[0.08,0.8*sat,bri,opac],
        Hue[0.17,0.6*sat,bri,opac],GrayLevel[bri,opac],Hue[0.95,0.4*sat,bri,opac],
        Hue[0.79,0.7*sat,bri,opac],Hue[0.74,sat,bri,opac],GrayLevel[1-sat,opac]},
        Mod[par,1]],
    colName==="Temperature",
      If[0<=Mod[par,1]<=0.5,
      Blend[{GrayLevel[1-sat,opac],Hue[0,sat,bri,opac],
       Hue[0.05,0.94*sat,bri,opac],Hue[0.1,0.85*sat,bri,opac],
       Hue[0.15,0.76*sat,bri,opac],Hue[0.2,0.67*sat,bri,opac],
       Hue[0.25,0.58*sat,bri,opac],Hue[0.3,0.49*sat,bri,opac],
       Hue[0.35,0.4*sat,bri,opac]},Mod[2*par,1]],
      Blend[{Hue[0.35,0.4*sat,bri,opac],Hue[0.4,0.49*sat,bri,opac],
       Hue[0.45,0.58*sat,bri,opac],Hue[0.5,0.67*sat,bri,opac],
       Hue[0.55,0.76*sat,bri,opac],Hue[0.6,0.85*sat,bri,opac],
       Hue[0.65,0.94*sat,bri,opac],Hue[0.7,sat,bri,opac],GrayLevel[1-sat,opac]},
       Mod[2*par,1]]],
    colName==="Flower",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[1,0,0,opac],RGBColor[1,0.2,0,opac],RGBColor[1,0.4,0,opac],
          RGBColor[1,0.6,0,opac],RGBColor[1,0.8,0,opac],RGBColor[1,1,0,opac],
          RGBColor[1,0.833725,0.2,opac],RGBColor[1,0.667451,0.4,opac],
          RGBColor[1,0.501176,0.6,opac],RGBColor[1,0.334902,0.8,opac],
          RGBColor[1,0.168627,1,opac],RGBColor[0.866667,0.168627,1,opac],
          RGBColor[0.733333,0.168627,1,opac],RGBColor[0.6,0.168627,1,opac],
          RGBColor[0.466667,0.168627,1,opac],RGBColor[0.333333,0.168627,1,opac],
          RGBColor[0.466667,0.134902,0.8,opac],RGBColor[0.6,0.101176,0.6,opac],
          RGBColor[0.733333,0.067451,0.4,opac],
          RGBColor[0.866667,0.033725,0.2,opac],RGBColor[1,0,0,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="Pansy",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[0.91804,0.08196,0.06666,opac],
          RGBColor[0.98549,0.22078,0,opac],RGBColor[1,0.38941,0,opac],
          RGBColor[1,0.56706,0,opac],RGBColor[1,0.72627,0,opac],
          RGBColor[1,0.87176,0.01020,opac],RGBColor[1,0.97608,0.13490,opac],
          RGBColor[0.94314,1,0.31098,opac],RGBColor[0.86745,1,0.48000,opac],
          RGBColor[0.78549,0.99451,0.66314,opac],
          RGBColor[0.70275,0.88627,0.81490,opac],
          RGBColor[0.62588,0.74314,0.93686,opac],RGBColor[0.57922,0.57922,1,opac],
          RGBColor[0.54392,0.40118,1,opac],RGBColor[0.54314,0.23216,0.98157,opac],
          RGBColor[0.55569,0.08667,0.88706,opac],RGBColor[0.60627,0,0.74392,opac],
          RGBColor[0.66314,0,0.58157,opac],RGBColor[0.74980,0,0.39804,opac],
          RGBColor[0.82980,0,0.22824,opac],RGBColor[0.90902,0.07294,0.07294,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="ComplexArgument",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[1,0.21373,0.17294,opac],RGBColor[1,0.4114,0.11569,opac],
          RGBColor[1,0.58353,0.03804,opac],RGBColor[1,0.71686,0,opac],
          RGBColor[0.98824,0.82000,0,opac],RGBColor[0.90235,0.91765,0,opac],
          RGBColor[0.71294,0.92706,0,opac],RGBColor[0.48588,0.93451,0,opac],
          RGBColor[0.30902,0.92078,0.11529,opac],RGBColor[0.21412,0.90745,0.30588,opac],
          RGBColor[0.09098,0.89137,0.45294,opac],RGBColor[0.12196,0.84588,0.61647,opac],
          RGBColor[0.10627,0.80471,0.77922,opac],RGBColor[0.15490,0.72078,0.88902,opac],
          RGBColor[0.22902,0.59098,0.96824,opac],RGBColor[0.25686,0.46078,1,opac],
          RGBColor[0.37255,0.36235,1,opac],RGBColor[0.45647,0.25255,1,opac],
          RGBColor[0.57020,0.16235,1,opac],RGBColor[0.70353,0.09176,0.93804,opac],
          RGBColor[0.81725,0,0.86824,opac],RGBColor[0.89098,0,0.71765,opac],
          RGBColor[0.95490,0,0.56235,opac],RGBColor[0.99412,0.05608,0.42824,opac],
          RGBColor[1,0.13922,0.30235,opac],RGBColor[1,0.21373,0.17294,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="MunsellColor",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[1,0,0,opac],RGBColor[1,0.282353,0.043137,opac],
          RGBColor[1,0.458824,0.101961,opac],RGBColor[1,0.611765,0.082353,opac],
          RGBColor[1,0.8,0,opac],RGBColor[1,0.901961,0,opac],
          RGBColor[0.784314,0.9490196,0,opac],RGBColor[0.627451,0.933333,0,opac],
          RGBColor[0.470588,0.8745098,0.141176,opac],
          RGBColor[0.054902,0.819608,0.266666,opac],
          RGBColor[0,0.709804,0.568627,opac],RGBColor[0,0.596078,0.768627,opac],
          RGBColor[0,0.466667,0.933333,opac],RGBColor[0,0.301961,1,opac],
          RGBColor[0.141176,0.141176,1,opac],RGBColor[0.341176,0,0.976471,opac],
          RGBColor[0.470588,0,0.901961,opac],RGBColor[0.552941,0,0.850980,opac],
          RGBColor[0.654902,0,0.815686,opac],RGBColor[0.729412,0,0.768627,opac],
          RGBColor[0.807843,0,0.686275,opac],RGBColor[0.8745098,0,0.611765,opac],
          RGBColor[0.933333,0,0.537255,opac],RGBColor[0.976471,0,0.341176,opac],
          RGBColor[1,0,0,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="Spectrum",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[0,0,0,opac],RGBColor[0.121569,0,0,opac],
          RGBColor[0.254902,0,0,opac],RGBColor[0.517647,0,0,opac],
          RGBColor[0.737255,0,0,opac],RGBColor[0.866667,0,0,opac],
          RGBColor[0.952941,0.019608,0,opac],RGBColor[0.980392,0.066667,0,opac],
          RGBColor[0.992157,0.219608,0,opac],RGBColor[0.992157,0.4509804,0,opac],
          RGBColor[0.933333,0.666667,0,opac],RGBColor[0.8509804,0.890196,0,opac],
          RGBColor[0.619608,0.956863,0,opac],RGBColor[0.364706,1,0,opac],
          RGBColor[0.184314,1,0,opac],RGBColor[0.031373,0.996078,0.011765,opac],
          RGBColor[0.011765,0.956863,0.078431,opac],
          RGBColor[0,0.8745098,0.219608,opac],RGBColor[0,0.686275,0.533333,opac],
          RGBColor[0.011765,0.466667,0.764706,opac],
          RGBColor[0.035294,0.211765,0.882353,opac],
          RGBColor[0.1490196,0.070588,0.858824,opac],
          RGBColor[0.286275,0.015686,0.733333,opac],
          RGBColor[0.239216,0,0.505882,opac],RGBColor[0.121569,0,0.247059,opac],
          RGBColor[0.058824,0,0.113725,opac],RGBColor[0,0,0,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="SkyRainbow",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[0.803922,0.8745098,1,opac],
          RGBColor[0.823529,0.780392,0.882353,opac],
          RGBColor[0.8509804,0.658824,0.745098,opac],
          RGBColor[0.905882,0.427451,0.486275,opac],
          RGBColor[0.9490196,0.235294,0.266667,opac],
          RGBColor[0.972549,0.117647,0.133333,opac],
          RGBColor[0.992157,0.062745,0.0509804,opac],
          RGBColor[0.996078,0.082353,0.019608,opac],
          RGBColor[1,0.223529,0.007843,opac],
          RGBColor[0.996078,0.4509804,0.003922,opac],
          RGBColor[0.937255,0.670588,0.003922,opac],
          RGBColor[0.8509804,0.890196,0,opac],
          RGBColor[0.619608,0.956863,0,opac],RGBColor[0.364706,1,0,opac],
          RGBColor[0.184314,1,0,opac],RGBColor[0.031373,0.996078,0.011765,opac],
          RGBColor[0.015686,0.960784,0.082353,opac],
          RGBColor[0.003922,0.878431,0.223529,opac],
          RGBColor[0.007843,0.690196,0.541176,opac],
          RGBColor[0.027451,0.482353,0.784314,opac],
          RGBColor[0.07450984,0.258824,0.933333,opac],
          RGBColor[0.258824,0.192157,0.996078,opac],
          RGBColor[0.501961,0.2509804,1,opac],RGBColor[0.635294,0.435294,1,opac],
          RGBColor[0.729412,0.666667,1,opac],RGBColor[0.768627,0.784314,1,opac],
          RGBColor[0.803922,0.8745098,1,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="Psychedelic",
      Module[{n=4},
      Blend[Join[Flatten[Table[
       Table[hueToRGB[m n/(2 n+1),sat,bri,opac],{Floor[Sqrt[n]]}],
        {m,0,2 n}]],{Hue[n,sat,bri,opac]}],Mod[par,1]]],
    colName==="SoftHue",
      softHueFunction[par,sat,bri,opac],
    colName==="HyperSoftHue",
      hyperSoftHueFunction[par,sat,bri,opac],
    colName==="NaturalHue",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[1,0.082352941,0.129411765,opac],
          RGBColor[1,0.250980392,0,opac],RGBColor[1,0.423529412,0,opac],
          RGBColor[1,0.6,0,opac],RGBColor[1,0.749019608,0,opac],
          RGBColor[1,0.901960784,0,opac],RGBColor[0.925490196,1,0,opac],
          RGBColor[0.823529412,1,0,opac],RGBColor[0.650980392,1,0,opac],
          RGBColor[0.423529412,1,0,opac],RGBColor[0.050980392,1,0,opac],
          RGBColor[0,0.956862745,0.337254902,opac],
          RGBColor[0,0.917647059,0.686274510,opac],
          RGBColor[0,0.870588235,0.890196078,opac],
          RGBColor[0,0.8,1,opac],RGBColor[0,0.725490196,1,opac],
          RGBColor[0,0.623529412,1,opac],
          RGBColor[0.082352941,0.494117647,1,opac],
          RGBColor[0.168627451,0.376470588,1,opac],
          RGBColor[0.266666667,0.305882353,1,opac],
          RGBColor[0.4,0.250980392,1,opac],
          RGBColor[0.525490196,0.207843137,1,opac],
          RGBColor[0.650980392,0.125490196,1,opac],
          RGBColor[0.760784314,0.043137255,1,opac],
          RGBColor[0.862745098,0,0.984313725,opac],
          RGBColor[0.941176471,0,0.968627451,opac],RGBColor[1,0,0.8,opac],
          RGBColor[1,0.015686275,0.556862745,opac],
          RGBColor[1,0.082352941,0.129411765,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="OilHue",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[0.8580392157,0.09843137255,0.09843137255,opac],
          RGBColor[0.916078431,0.222745098,0.0494117647,opac],
          RGBColor[0.9266666667,0.3607843137,0.04980392155,opac],
          RGBColor[0.9490196078,0.4749019608,0.0466666667,opac],
          RGBColor[0.9349019608,0.608627451,0.068235294,opac],
          RGBColor[0.8988235294,0.7152941176,0.1545098039,opac],
          RGBColor[0.8498039216,0.819215686,0.2329411765,opac],
          RGBColor[0.745098039,0.8470588235,0.337254902,opac],
          RGBColor[0.650588235,0.8545098039,0.4007843137,opac],
          RGBColor[0.531372549,0.8780392157,0.414117647,opac],
          RGBColor[0.428235294,0.8717647059,0.4843137255,opac],
          RGBColor[0.2905882353,0.84627451,0.596470588,opac],
          RGBColor[0.197254902,0.8039215686,0.696862745,opac],
          RGBColor[0.1011764706,0.7537254902,0.7843137255,opac],
          RGBColor[0.0552941176,0.6517647059,0.830196078,opac],
          RGBColor[0.0188235294,0.559215686,0.865098039,opac],
          RGBColor[0.014117647,0.442352941,0.8807843137,opac],
          RGBColor[0.0537254902,0.3341176471,0.859215686,opac],
          RGBColor[0.1149019608,0.2219607843,0.808235294,opac],
          RGBColor[0.187843137,0.1266666667,0.7639215686,opac],
          RGBColor[0.3062745098,0.0717647059,0.688627451,opac],
          RGBColor[0.399215686,0.0576470588,0.5980392157,opac],
          RGBColor[0.5184313725,0.0545098039,0.462352941,opac],
          RGBColor[0.6235294118,0.0780392157,0.353333333,opac],
          RGBColor[0.7384313725,0.1317647059,0.2082352941,opac],
          RGBColor[0.8580392157,0.09843137255,0.09843137255,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="TropicalHue",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[0.9815686275,0.237254902,0.2321568627,opac],
          RGBColor[1,0.2458823529,0.0521568627,opac],
          RGBColor[1,0.262352941,0,opac],RGBColor[1,0.3662745098,0,opac],
          RGBColor[1,0.465098039,0,opac],RGBColor[1,0.5588235294,0,opac],
          RGBColor[1,0.662352941,0,opac],RGBColor[1,0.7443137255,0,opac],
          RGBColor[0.9207843137,0.7474509804,0,opac],
          RGBColor[0.736078431,0.756470588,0.017254902,opac],
          RGBColor[0.5603921569,0.7643137255,0.1066666667,opac],
          RGBColor[0.38,0.7725490196,0.196470588,opac],
          RGBColor[0.23843137255,0.7737254902,0.325098039,opac],
          RGBColor[0.148627451,0.765490196,0.505490196,opac],
          RGBColor[0.0635294118,0.756862745,0.685490196,opac],
          RGBColor[0,0.748627451,0.870980392,opac],
          RGBColor[0,0.7407843137,1,opac],RGBColor[0,0.676470588,1,opac],
          RGBColor[0,0.5776470588,1,opac],RGBColor[0,0.4788235294,1,opac],
          RGBColor[0,0.38,1,opac],RGBColor[0,0.2811764706,1,opac],
          RGBColor[0,0.2470588235,1,opac],
          RGBColor[0.1725490196,0.2388235294,1,opac],
          RGBColor[0.3529411765,0.2305882353,0.934117647,opac],
          RGBColor[0.5380392157,0.2219607843,0.8439215686,opac],
          RGBColor[0.7137254902,0.214117647,0.7545098039,opac],
          RGBColor[0.807843137,0.2215686275,0.5784313725,opac],
          RGBColor[0.8976470588,0.2298039216,0.3980392157,opac],
          RGBColor[0.9815686275,0.237254902,0.2321568627,opac]},
          Mod[par,1]]},sat]},bri],
    colName==="Season",
      Blend[{RGBColor[0,0,0,opac],
      Blend[{RGBColor[1,1,1,opac],
        Blend[{RGBColor[1,0.266666667,0.635294118,opac],
          RGBColor[1,0.482352941,0.615686275,opac],
          RGBColor[1,0.674509804,0.580392157,opac],
          RGBColor[1,0.815686275,0.521568627,opac],
          RGBColor[0.968627451,0.917647059,0.368627451,opac],
          RGBColor[0.917647059,1,0.168627451,opac],
          RGBColor[0.717647059,1,0.168627451,opac],
          RGBColor[0.486274510,0.976470588,0.133333333,opac],
          RGBColor[0.184313725,0.901960784,0.031372549,opac],
          RGBColor[0.050980392,0.827450980,0.043137255,opac],
          RGBColor[0,0.756862745,0.113725490,opac],
          RGBColor[0.243137255,0.717647059,0.074509804,opac],
          RGBColor[0.462745098,0.666666667,0.066666667,opac],
          RGBColor[0.635294118,0.592156863,0.121568627,opac],
          RGBColor[0.815686275,0.494117647,0.117647059,opac],
          RGBColor[1,0.384313725,0.082352941,opac],
          RGBColor[1,0.313725490,0.172549020,opac],
          RGBColor[0.972549020,0.278431373,0.298039216,opac],
          RGBColor[0.878431373,0.309803922,0.482352941,opac],
          RGBColor[0.772549020,0.376470588,0.639215686,opac],
          RGBColor[0.662745098,0.462745098,0.780392157,opac],
          RGBColor[0.619607843,0.552941176,0.858823529,opac],
          RGBColor[0.560784314,0.635294118,0.925490196,opac],
          RGBColor[0.474509804,0.686274510,0.972549020,opac],
          RGBColor[0.505882353,0.650980392,0.988235294,opac],
          RGBColor[0.541176471,0.6,1,opac],
          RGBColor[0.603921569,0.525490196,0.992156863,opac],
          RGBColor[0.678431373,0.454901961,0.964705882,opac],
          RGBColor[0.776470588,0.396078431,0.905882353,opac],
          RGBColor[0.886274510,0.333333333,0.784313725,opac],
          RGBColor[1,0.266666667,0.635294118,opac]},
          Mod[par,1]]},sat]},bri]];

(********************************************************)

hueToRGB[h_,s_,v_,a_]:=
  Module[{col},
   col=RGBColor[trigRGB[2 Pi*h],trigRGB[2 Pi*(h-1/3)],trigRGB[2 Pi*(h+1/3)]];
   Opacity[a,Darker[Lighter[col,1-s],1-v]]];

trigRGB[h_]:=
  Module[{x=Mod[h,2 Pi]},
    Which[0<=x<Pi/3 || 5 Pi/3<=x<2 Pi,1,
      2 Pi/3<=x<4 Pi/3,0,
      Pi/3<=x<2 Pi/3,2-3/Pi*x,
      4 Pi/3<=x<5 Pi/3,3/Pi*x-4]];

(********************************************************)

softHueFunction[h_,s_,v_,a_]:=
  Module[{trig,max,min},
   trig={1+Cos[2 Pi*h],1+Cos[2 Pi*(h-1/3)],1+Cos[2 Pi*(h+1/3)]}/2;
   max=Max[trig];
   min=Min[trig];
   Hue[h,s*(max-min)/max,v*max,a]];

(********************************************************)

hyperSoftHueFunction[h_,s_,v_,a_]:=
  Module[{trig,max,min,n=3},
   trig={Log[n+1,n (1+Cos[2 Pi*h])/2+1],Log[n+1,n (1+Cos[2 Pi (h-1/3)])/2+1],
         Log[n+1,n (1+Cos[2 Pi (h+1/3)])/2+1]};
   max=Max[trig];
   min=Min[trig];
   Hue[h,s*(max-min)/max,v*max,a]];



(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*  (1) AbsArgPlot3D, ArgDensityPlot,          *)
(*      AbsArgArcSinhPlot3D,                   *)
(*      FundamentalRegionPlot,                 *)
(*      FundamentalRegionPlot3D,               *) 
(*      RiemannSpherePlot3D,                   *) 
(*      RiemannSphereDensityPlot,              *) 
(*      RiemannSphereArcSinhPlot3D             *)
(*                                             *)
(*  (2) AbsArgLogPlot3D,                       *)
(*      RiemannSphereLogPlot3D,                *) 
(***********************************************)

absContoursCol[colL_,pRangMax_,a_]:=
  If[Not[VectorQ[colL]],colL,
    If[Length[colL]===1,colL[[1]],
      Blend[colL,
       If[a>pRangMax,1,a/pRangMax]]]];

absContoursCol[colL_,{pRangMin_,pRangMax_},a_]:=
  If[Not[VectorQ[colL]],colL,
    If[Length[colL]===1,colL[[1]],
      Blend[colL,
       If[a<pRangMin,0,
        If[a>pRangMax,1,(a-pRangMin)/(pRangMax-pRangMin)]]]]];



(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*    AbsArgPlot3D, ArgDensityPlot,            *)
(*    AbsArgLogPlot3D, AbsArgArcSinhPlot3D,    *)
(*    FundamentalRegionPlot,                   *)
(*    FundamentalRegionPlot3D,                 *)
(*    TopographyPlot3D, RiemannSpherePlot3D,   *) 
(*    RiemannSphereDensityPlot,                *) 
(*    RiemannSphereLogPlot3D,                  *) 
(*    RiemannSphereArcSinhPlot3D,              *)
(*    NonMeshAbsArgPlot3D,                     *) 
(*    NonMeshArgDensityPlot                    *)
(***********************************************)

origMeshF[funcName_String,base_,i_]:=
  Which[
    funcName==="Log",
      base^i,
    funcName==="Linear",
      If[i<0,1/(1-(base-1)*i),1+(base-1)*i],
    funcName==="LinearLog",
      If[i<0,Exp[(base-1)*i],1+(base-1)*i],
    funcName==="LogLinear",
      If[i<0,1/(1-(base-1)*i),base^i],
    funcName==="LogLog",
      If[i<0,base^(1-base^(-i)),base^(base^i-1)],
    funcName==="LogShift1",
      1+base^i]//N;

origMeshInvF[funcName_String,base_,a_]:=
  Which[
    funcName==="Log",
      Log[base,a],
    funcName==="Linear",
      If[a<1,(a-1)/(base-1)/a,(a-1)/(base-1)],
    funcName==="LinearLog",
      If[a<1,Log[a]/(base-1),(a-1)/(base-1)],
    funcName==="LogLinear",
      If[a<1,(a-1)/(base-1)/a,Log[base,a]],
    funcName==="LogLog",
      If[a<1,-Log[base,1-Log[base,a]],Log[base,1+Log[base,a]]],
    funcName==="LogShift1",
      Log[base,a-1]]//N;



(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*            RePlot3D, ImPlot3D               *)
(***********************************************)

origMeshReIm[funcName_String,{initv_,base_},i_Integer]:=
  0/;(N[i]===0.)

origMeshReIm[funcName_String,{initv_,base_},i_Integer]:=
  -origMeshReIm[funcName,{initv,base},-i]/;(i<0)

origMeshReIm[funcName_String,{initv_,base_},i_Integer]:=
  Which[
    funcName==="Logarithm",
      initv*base^(i-1),
    funcName==="Linear",
      initv+base*(i-1)]/;(i>0)

arcOMReIm[funcName_String,{initv_,base_},pRange_]:=
  0/;(N[pRange]===0.)

arcOMReIm[funcName_String,{initv_,base_},pRange_]:=
  -arcOMReIm[funcName,{initv,base},-pRange]/;(pRange<0)

arcOMReIm[funcName_String,{initv_,base_},pRange_]:=
  Which[
    funcName==="Logarithm",
      Log[base,pRange/initv]+1,
    funcName==="Linear",
      (pRange-initv)/base+1]/;(pRange>0)



(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*             TopographyPlot3D                *)
(***********************************************)

landColorFunc[colName_String,par_,{sat_,bri_,opac_}]:=
  Which[
    colName==="geography",
      Blend[{Hue[0.7,0.5*sat,0.7*bri,opac],Hue[0.6,0.5*sat,0.8*bri,opac],
        Hue[0.57,0.5*sat,0.9*bri,opac],Hue[0.55,0.5*sat,bri,opac],
        Hue[0.4,0.5*sat,0.8*bri,opac],Hue[0.35,0.5*sat,0.9*bri,opac],
        Hue[0.3,0.5*sat,bri,opac],Hue[0.05,0.4*sat,0.8*bri,opac]},
        Mod[par,1]],
    colName==="CoralReef",
      Blend[{Hue[0.7,0.5*sat,0.7*bri,opac],Hue[0.55,0.4*sat,0.8*bri,opac],
        Hue[0.45,0.3*sat,0.9*bri,opac],Hue[0.2,0.2*sat,bri,opac],
        Hue[0.4,0.9*sat,0.7*bri,opac],Hue[0.35,0.8*sat,0.6*bri,opac],
        Hue[0.3,0.7*sat,0.5*bri,opac],Hue[0.05,0.7*sat,0.5*bri,opac]},
        Mod[par,1]],
    colName==="SnowCap",
      Blend[{Hue[0.7,0.5*sat,0.7*bri,opac],Hue[0.63,0.5*sat,0.7*bri,opac],
        Hue[0.6,0.5*sat,0.8*bri,opac],Hue[0.57,0.5*sat,0.9*bri,opac],
        Hue[0.55,0.5*sat,bri,opac],Hue[0.4,0.5*sat,0.8*bri,opac],
        Hue[0.35,0.5*sat,0.9*bri,opac],Hue[0.3,0.5*sat,bri,opac],
        Hue[0.05,0.4*sat,0.8*bri,opac],GrayLevel[bri,opac]},Mod[par,1]],
    colName==="Sunrise",
      Blend[{Hue[0.7,0.8*sat,0.4*bri,opac],Hue[0.6,0.7*sat,0.5*bri,opac],
        Hue[0.55,0.6*sat,0.6*bri,opac],Hue[0.5,0.5*sat,0.7*bri,opac],
        Hue[0.2,0.5*sat,0.8*bri,opac],Hue[0.15,0.5*sat,0.9*bri,opac],
        Hue[0.1,0.6*sat,bri,opac],Hue[0.05,0.8*sat,bri,opac]},
        Mod[par,1]]];



(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*         RiemannSpherePlot3D,                *) 
(*         RiemannSphereDensityPlot,           *) 
(*         RiemannSphereLogPlot3D,             *) 
(*         RiemannSphereArcSinhPlot3D          *)
(***********************************************)

rsTypeCot[z_,typ_]:=If[typ===1,Cot[z],Tan[z]];

rsTypeThetaMin[r_,typ_]:=If[typ===1,2*ArcCot[r],0.00000000001];

rsTypeThetaMax[r_,typ_]:=If[typ===1,3.14159265358,2*ArcTan[r]];



(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*  NonMeshAbsArgPlot3D, NonMeshArgDensityPlot *)
(***********************************************)

colorLevelFP[pRangeMax_,a_]:=If[a>pRangeMax,1,a/pRangeMax];

colorLevelFN[pRangeMax_,a_]:=If[a>pRangeMax,0,1-a/pRangeMax];

opacityLevelF[{fu_String,b_},a_,opa_]:=
  opa*Mod[Ceiling[origMeshInvF[fu,b,a]],2];

scaleSat[{fu_String,b_},prMax_,a_,opa_]:=
  1-colorLevelFP[prMax,a]*opacityLevelF[{fu,b},a,opa];

scaleBri[{fu_String,b_},prMax_,a_,opa_]:=
  1-colorLevelFN[prMax,a]*opacityLevelF[{fu,b},a,opa];



(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*    AbsArgPlot3D, AbsArgLogPlot3D,           *)
(*    AbsArgArcSinhPlot3D,                     *)
(*    FundamentalRegionPlot3D,                 *)
(*    TopographyPlot3D, RiemannSpherePlot3D,   *)
(*    RiemannSphereDensityPlot,                *) 
(*    RiemannSphereLogPlot3D,                  *) 
(*    RiemannSphereArcSinhPlot3D               *)
(***********************************************)

meshOption3Take[{opt1_,opt2_},{a_,b_,c_}]:=
  Module[{test1,test2},
    {test1,test2}=Map[MemberQ[{False,None},#]&,{opt1,opt2}];
    Which[test1 && Not[test2],{b,c},
      Not[test1] && test2,{a},
      test1 && test2,{},
      Not[test1] && Not[test2],{a,b,c}]];

(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*            RePlot3D, ImPlot3D               *)
(***********************************************)

meshOption2Take[{opt1_,opt2_},{a_,b_}]:=
  Module[{test1,test2},
    {test1,test2}=Map[MemberQ[{False,None},#]&,{opt1,opt2}];
    Which[test1 && Not[test2],{b},
      Not[test1] && test2,{a},
      test1 && test2,{},
      Not[test1] && Not[test2],{a,b}]];

(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*              ArgDensityPlot                 *)
(***********************************************)

plot4Take[{opt1_,opt2_},{plot1_,plot2_,plot3_,plot4_}]:=
  Module[{test1,test2},
    {test1,test2}=Map[MemberQ[{False,None},#]&,{opt1,opt2}];
    Which[test1 && Not[test2],{plot1,plot2,plot3},
      Not[test1] && test2,{plot1,plot4},
      test1 && test2,{plot1},
      Not[test1] && Not[test2],{plot1,plot2,plot3,plot4}]];

(***********************************************)
(*        Using in plot functions name:        *)
(*                                             *)
(*           FundamentalRegionPlot             *)
(***********************************************)

plot3Take[{opt1_,opt2_},{plot1_,plot2_,plot3_}]:=
  Module[{test1,test2},
    {test1,test2}=Map[MemberQ[{False,None},#]&,{opt1,opt2}];
    Which[test1 && Not[test2],{plot1,plot2},
      Not[test1] && test2,{plot1,plot3},
      test1 && test2,{plot1},
      Not[test1] && Not[test2],{plot1,plot2,plot3}]];



(****************************** GridLinesPlot ******************************)

Options[GridLinesPlot]=
  Join[{GridLinesColor->Automatic, 
        FundamentalRectangle->Automatic},Options[Plot]];

GridLinesPlot[unknownFunc_,{x_,xmin_,xmax_},OptionsPattern[]]:=
  Module[{plotrange,gridlinescolor,fundamentalrectangle},
    plotrange=OptionValue[PlotRange];
    gridlinescolor=OptionValue[GridLinesColor];
    fundamentalrectangle=OptionValue[FundamentalRectangle];
     If[gridlinescolor===Automatic,gridlinescolor={Hue[0.5],Hue[0.5]}];
     If[fundamentalrectangle===Automatic,fundamentalrectangle={{0,0},{1,1}}];
  Plot[unknownFunc,{x,xmin,xmax},
    Exclusions->{{1/unknownFunc==0,
                  Not[plotrange[[1]]<=Re[unknownFunc]<=plotrange[[2]]]}},
    GridLines->
     {gridltable[{fundamentalrectangle[[1,1]],fundamentalrectangle[[2,1]]},
      {xmin,xmax},gridlinescolor[[1]]],
      gridltable[{fundamentalrectangle[[1,2]],fundamentalrectangle[[2,2]]},
      {plotrange[[1]],plotrange[[2]]}, gridlinescolor[[2]]]},
    PlotRegion->{{0.02,0.99},{0.02,0.99}},
    Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot]]]]]]

GridLinesPlot[f_List,{x_,xmin_,xmax_},OptionsPattern[]]:=
  Module[{graphicsList,plotstyle},
    plotstyle=OptionValue[PlotStyle];
    graphicsList=MapThread[GridLinesPlot[#1,{x,xmin,xmax},PlotStyle->#2,
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[GridLinesPlot]]]]]&,
      {f,plotstyle}];
    Show[graphicsList]]



(************************* GridLinesParametricPlot *************************)

Options[GridLinesParametricPlot]=
  Join[{GridLinesColor->Automatic, 
        FundamentalRectangle->Automatic},Options[ParametricPlot]];

GridLinesParametricPlot[{unknownF1_,unknownF2_},{t_,tmin_,tmax_},OptionsPattern[]]:=
  Module[{plotrange,gridlinescolor,fundamentalrectangle},
    plotrange=OptionValue[PlotRange];
    gridlinescolor=OptionValue[GridLinesColor];
    fundamentalrectangle=OptionValue[FundamentalRectangle];
     If[gridlinescolor===Automatic,gridlinescolor={Hue[0.5],Hue[0.5]}];
     If[fundamentalrectangle===Automatic,fundamentalrectangle={{0,0},{1,1}}];
   ParametricPlot[{unknownF1,unknownF2},{t,tmin,tmax},
     GridLines->
      {gridltable[{fundamentalrectangle[[1,1]],fundamentalrectangle[[2,1]]},
      {plotrange[[1,1]],plotrange[[1,2]]},gridlinescolor[[1]]],
      gridltable[{fundamentalrectangle[[1,2]],fundamentalrectangle[[2,2]]},
      {plotrange[[2,1]],plotrange[[2,2]]},gridlinescolor[[2]]]},
     PlotRegion->{{0.02,0.99},{0.02,0.99}},
    Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[ParametricPlot]]]]]]



(******************************* AbsArgPlot ********************************)

Options[AbsArgPlot]=
  Join[{GridLinesColor->Automatic, 
        FundamentalRectangle->Automatic, 
        ArgColorScheme->Automatic},Options[Plot]];

AbsArgPlot[unknownFunc_,{x_,xmin_,xmax_},OptionsPattern[]]:=
  Module[{u,fillPl,linePl,plotrange,gridlinescolor,fundamentalrectangle,argcolorscheme},
    plotrange=OptionValue[PlotRange];
    gridlinescolor=OptionValue[GridLinesColor];
    fundamentalrectangle=OptionValue[FundamentalRectangle];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
     If[gridlinescolor===Automatic,gridlinescolor={GrayLevel[0.7],GrayLevel[0.7]}];
     If[fundamentalrectangle===Automatic,fundamentalrectangle={{0,0},{1,1}}];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,0.6}}];
   fillPl=Plot[Abs[unknownFunc],{x,xmin,xmax},
     ColorFunction->Function[{u,y},origColorScheme[argcolorscheme[[1]],
       Arg[unknownFunc/.x->u]/(2*Pi),argcolorscheme[[2]]]],
     ColorFunctionScaling->False,Filling->Axis,
     GridLines->
      {gridltable[{fundamentalrectangle[[1,1]],fundamentalrectangle[[2,1]]},
       {xmin,xmax},gridlinescolor[[1]]],
       gridltable[{fundamentalrectangle[[1,2]],fundamentalrectangle[[2,2]]},
       {plotrange[[1]],plotrange[[2]]}, gridlinescolor[[2]]]},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot]]]]];
   linePl=Plot[Abs[unknownFunc],{x,xmin,xmax},
     ColorFunction->Function[{u,y},
       origColorScheme[argcolorscheme[[1]],Arg[unknownFunc/.x->u]/(2*Pi),
       {argcolorscheme[[2,1]],argcolorscheme[[2,2]],1}]],
     ColorFunctionScaling->False,
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot]]]]];
   Show[fillPl,linePl,PlotRegion->{{0.02,0.99},{0.02,0.99}}]]



(******************************** ReImPlot *********************************)

Options[ReImPlot]=
  Join[{GridLinesColor->Automatic, 
        FundamentalRectangle->Automatic, 
        ReImFillingColors->Automatic, 
        ZeroPointsStyle->Automatic},Options[Plot]];

ReImPlot[unknownFunc_,{x_,xmin_,xmax_},OptionsPattern[]]:=
  Module[{rePl,imPl,plotrange,gridlinescolor,fundamentalrectangle,
          reimfillcolors,zeropointstyle},
    plotrange=OptionValue[PlotRange];
    gridlinescolor=OptionValue[GridLinesColor];
    fundamentalrectangle=OptionValue[FundamentalRectangle];
    reimfillcolors=OptionValue[ReImFillingColors];
    zeropointstyle=OptionValue[ZeroPointsStyle];
     If[gridlinescolor===Automatic,gridlinescolor={GrayLevel[0.7],GrayLevel[0.7]}];
     If[fundamentalrectangle===Automatic,fundamentalrectangle={{0,0},{1,1}}];
     If[reimfillcolors===Automatic,
        reimfillcolors={{Hue[0,1,1,0.6],Hue[0.5,1,1,0.6]},
                        {Hue[0.25,1,1,0.6],Hue[0.75,1,1,0.6]}}];
     If[zeropointstyle===Automatic,
        zeropointstyle={PointSize[Large],Hue[0],Opacity[0.5]}];
   rePl=Plot[Re[unknownFunc],{x,xmin,xmax},Filling->Axis,
     FillingStyle->Reverse[reimfillcolors[[1]]],
     GridLines->
       {gridltable[{fundamentalrectangle[[1,1]],fundamentalrectangle[[2,1]]},
        {xmin,xmax},gridlinescolor[[1]]],
        gridltable[{fundamentalrectangle[[1,2]],fundamentalrectangle[[2,2]]},
        {plotrange[[1]],plotrange[[2]]}, gridlinescolor[[2]]]},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot]]]]];
   imPl=Plot[Im[unknownFunc],{x,xmin,xmax},Filling->Axis,
     FillingStyle->Reverse[reimfillcolors[[2]]],
     Exclusions->{{Re[unknownFunc]==0,Abs[unknownFunc]<0.01},
                {Im[unknownFunc]==0,Abs[unknownFunc]<0.01}},
     ExclusionsStyle->
       {Apply[Directive,zeropointstyle],Apply[Directive,zeropointstyle]},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot]]]]];
   Show[rePl,imPl,PlotRegion->{{0.02,0.99},{0.02,0.99}}]]



(****************************** AbsArgPlot3D *******************************)

Options[AbsArgPlot3D]=
  Join[{AbsContours->Automatic, 
        ArgContours->Automatic, 
        ArgColorScheme->Automatic, 
        ContoursColor->Automatic, 
        ContoursThickness->Automatic, 
        ExclusionsType->Automatic, 
        BranchCutColor->Automatic, 
        SurfaceProperty->Automatic,
        DefinitionDomain->Automatic},Options[Plot3D]];

AbsArgPlot3D[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,plotrange,boundstyle,abscontour,argcontour,argcolorscheme,
          contourcolor,contourthick,excluType,branchcolor,surfaceprop,defindomain,
          absC1,absC2,absC3,argC1,maxAbsContour},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    plotrange=OptionValue[PlotRange];
    boundstyle=OptionValue[BoundaryStyle];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    excluType=OptionValue[ExclusionsType];
    branchcolor=OptionValue[BranchCutColor];
    surfaceprop=OptionValue[SurfaceProperty];
    defindomain=OptionValue[DefinitionDomain];
     If[abscontour===Automatic,abscontour={"Log",2,-5}];
     If[argcontour===Automatic,argcontour=8];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,1}}];
     If[contourcolor===Automatic,
        contourcolor={{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[excluType===Automatic,excluType=Automatic];
     If[branchcolor===Automatic,branchcolor=GrayLevel[0]];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
     If[boundstyle===GrayLevel[0],boundstyle=Directive[GrayLevel[0],Thickness[0.0005]]];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,-5},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    maxAbsContour=Ceiling[origMeshInvF[absC1,absC2,plotrange[[2]]]];
   Plot3D[Min[Abs[f[x,y]],(1-10^-12)*plotrange[[2]]],
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     BoundaryStyle->boundstyle,
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     Exclusions->If[Or[excluType===Automatic,excluType===None],
       None,excluType/.z->(x+I y)],
     ExclusionsStyle->
       {Directive[branchcolor,Opacity[0.6]],Directive[branchcolor,Thickness[0.0006]]},
     ColorFunction->Function[{x,y,w},Directive[Join[{origColorScheme[argcolorscheme[[1]],
       Arg[f[x,y]]/(2*Pi),argcolorscheme[[2]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {#3&,Function[{x,y,w},Abs[Arg[f[x,y]]]],
       Function[{x,y,w},Im[f[x,y]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
       {Table[{origMeshF[absC1,N[absC2],i],
       absContoursCol[contourcolor[[1]],plotrange,
       origMeshF[absC1,absC2,i]]},
       {i,absC3,maxAbsContour}],
      Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
      {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
     PlotRegion->{{0.01,0.99},{0.01,0.99}},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]]]



(***************************** ArgDensityPlot ******************************)

Options[ArgDensityPlot]=
  Join[{AbsContours->Automatic, 
        ArgContours->Automatic, 
        ArgColorScheme->Automatic, 
        ContoursColor->Automatic, 
        ContoursThickness->Automatic, 
        ContoursColorAbsMax->Automatic,
        DefinitionDomain->Automatic},Options[DensityPlot],Options[ContourPlot]];

ArgDensityPlot[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,abscontour,argcontour,argcolorscheme,contourcolor,contourthick,
          cColorAbsMax,defindomain,absC1,absC2,absC3,argC1},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    cColorAbsMax=OptionValue[ContoursColorAbsMax];
    defindomain=OptionValue[DefinitionDomain];
     If[abscontour===Automatic,abscontour={"Log",2,{-5,30}}];
     If[argcontour===Automatic,argcontour=8];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,1}}];
     If[contourcolor===Automatic,
        contourcolor={{GrayLevel[0,0.8],GrayLevel[1,0.8]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.001],Thickness[0.001]}];
     If[cColorAbsMax===Automatic,cColorAbsMax=20];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,{-5,30}},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
   Show[plot4Take[{abscontour,argcontour},
   {DensityPlot[Arg[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
      PlotRange->{-Pi,Pi},AspectRatio->Automatic,Exclusions->None,
      ColorFunction->
        (origColorScheme[argcolorscheme[[1]],#/(2*Pi),argcolorscheme[[2]]]&),
      ColorFunctionScaling->False,
      RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
      Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[DensityPlot]]]]],
    ContourPlot[Abs[Arg[f[x,y]]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
      PlotRange->All,AspectRatio->Automatic,Exclusions->None,ContourShading->None,
      Contours->Table[{j*Pi/argC1,Directive[contourcolor[[2]],contourthick[[2]]]},
        {j,1,argC1-1}],
      RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
      Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[ContourPlot]]]]],
    ContourPlot[Im[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
      PlotRange->All,AspectRatio->Automatic,Exclusions->None,ContourShading->None,
      Contours->{{0,Directive[contourcolor[[2]],contourthick[[2]]]}},
      RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
      Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[ContourPlot]]]]],
    ContourPlot[Abs[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
      PlotRange->All,AspectRatio->Automatic,Exclusions->None,ContourShading->None,
      Contours->Table[{origMeshF[absC1,N[absC2],i],
        Directive[absContoursCol[contourcolor[[1]],cColorAbsMax,
        origMeshF[absC1,absC2,i]],contourthick[[1]]]},
       {i,absC3[[1]],absC3[[2]]}],
      RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
      Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[ContourPlot]]]]]}],
    PlotRegion->{{0.01,0.98},{0.01,0.97}}]]



(***************************** AbsArgLogPlot3D *****************************)

Options[AbsArgLogPlot3D]=
  Join[{AbsContours->Automatic, 
        ArgContours->Automatic, 
        ArgColorScheme->Automatic, 
        ContoursColor->Automatic, 
        ContoursThickness->Automatic, 
        ExclusionsType->Automatic, 
        BranchCutColor->Automatic, 
        TicksExpStep->Automatic, 
        SurfaceProperty->Automatic,
        DefinitionDomain->Automatic},Options[Plot3D]];

AbsArgLogPlot3D[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,plotrange,boundstyle,ticksXY,abscontour,argcontour,argcolorscheme,contourcolor,
          contourthick,excluType,branchcolor,ticksexpstep,surfaceprop,defindomain,
          absC1,absC2,argC1,minAbsContour,maxAbsContour},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    plotrange=OptionValue[PlotRange];
    boundstyle=OptionValue[BoundaryStyle];
    ticksXY=OptionValue[Ticks];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    excluType=OptionValue[ExclusionsType];
    branchcolor=OptionValue[BranchCutColor];
    ticksexpstep=OptionValue[TicksExpStep];
    surfaceprop=OptionValue[SurfaceProperty];
    defindomain=OptionValue[DefinitionDomain];
     If[abscontour===Automatic,abscontour={"Log",2}];
     If[argcontour===Automatic,argcontour=8];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,1}}];
     If[contourcolor===Automatic,
        contourcolor={{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[excluType===Automatic,excluType=Automatic];
     If[branchcolor===Automatic,branchcolor=GrayLevel[0]];
     If[ticksexpstep===Automatic,ticksexpstep=1];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
     If[boundstyle===GrayLevel[0],boundstyle=Directive[GrayLevel[0],Thickness[0.0005]]];
    {absC1,absC2}=If[MemberQ[{False,None},abscontour],{"Log",2},abscontour[[1;;2]]];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    minAbsContour=Floor[origMeshInvF[absC1,absC2,plotrange[[1]]]];
    maxAbsContour=Ceiling[origMeshInvF[absC1,absC2,plotrange[[2]]]];
   Plot3D[Min[Log[10,Abs[f[x,y]]],(1-10^-12)*Log[10,plotrange[[2]]]],
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     BoundaryStyle->boundstyle,
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     Exclusions->If[Or[excluType===Automatic,excluType===None],
       None,excluType/.z->(x+I y)],
     ExclusionsStyle->
       {Directive[branchcolor,Opacity[0.6]],Directive[branchcolor,Thickness[0.0006]]},
     ColorFunction->Function[{x,y,w},Directive[Join[{origColorScheme[argcolorscheme[[1]],
       Arg[f[x,y]]/(2*Pi),argcolorscheme[[2]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {#3&,Function[{x,y,w},Abs[Arg[f[x,y]]]],
        Function[{x,y,w},Im[f[x,y]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
       {Table[{Log[10,origMeshF[absC1,N[absC2],i]],
        absContoursCol[contourcolor[[1]],{Log[10,plotrange[[1]]],Log[10,plotrange[[2]]]},
        Log[10,origMeshF[absC1,absC2,i]]]},
        {i,minAbsContour,maxAbsContour}],
       Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
       {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     PlotRange->{Log[10,plotrange[[1]]],Log[10,plotrange[[2]]]},
     Ticks->{If[Head[ticksXY]===List,ticksXY[[1]],ticksXY],
       If[Head[ticksXY]===List,ticksXY[[2]],ticksXY],
       Join[{{0,1,{0.01,0}}},
       Table[{t,N[10^t],{0.005,0}},
       {t,ticksexpstep,Floor[Log[10,plotrange[[2]]]],ticksexpstep}],
       Table[{t,N[10^t],{0.005,0}},
       {t,-ticksexpstep,Ceiling[Log[10,plotrange[[1]]]],-ticksexpstep}]]},
     RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
     PlotRegion->{{0.01,0.99},{0.01,0.99}},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]]]



(*************************** AbsArgArcSinhPlot3D ***************************)

Options[AbsArgArcSinhPlot3D]=
  Join[{AbsContours->Automatic, 
        ArgContours->Automatic, 
        ArgColorScheme->Automatic, 
        ContoursColor->Automatic, 
        ContoursThickness->Automatic, 
        ExclusionsType->Automatic, 
        BranchCutColor->Automatic, 
        TicksExpStep->Automatic, 
        SurfaceProperty->Automatic,
        DefinitionDomain->Automatic},Options[Plot3D]];

AbsArgArcSinhPlot3D[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,plotrange,boundstyle,ticksXY,abscontour,argcontour,argcolorscheme,contourcolor,
          contourthick,excluType,branchcolor,ticksexpstep,surfaceprop,defindomain,
          absC1,absC2,absC3,argC1,maxAbsContour},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    plotrange=OptionValue[PlotRange];
    boundstyle=OptionValue[BoundaryStyle];
    ticksXY=OptionValue[Ticks];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    excluType=OptionValue[ExclusionsType];
    branchcolor=OptionValue[BranchCutColor];
    ticksexpstep=OptionValue[TicksExpStep];
    surfaceprop=OptionValue[SurfaceProperty];
    defindomain=OptionValue[DefinitionDomain];
     If[abscontour===Automatic,abscontour={"Log",2,-5}];
     If[argcontour===Automatic,argcontour=8];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,1}}];
     If[contourcolor===Automatic,
        contourcolor={{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[excluType===Automatic,excluType=Automatic];
     If[branchcolor===Automatic,branchcolor=GrayLevel[0]];
     If[ticksexpstep===Automatic,ticksexpstep=1];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
     If[boundstyle===GrayLevel[0],boundstyle=Directive[GrayLevel[0],Thickness[0.0005]]];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,-5},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    maxAbsContour=Ceiling[origMeshInvF[absC1,absC2,plotrange[[2]]]];
   Plot3D[Min[ArcSinh[Abs[f[x,y]]],(1-10^-12)*ArcSinh[plotrange[[2]]]],
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     BoundaryStyle->boundstyle,
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     Exclusions->If[Or[excluType===Automatic,excluType===None],
       None,excluType/.z->(x+I y)],
     ExclusionsStyle->
       {Directive[branchcolor,Opacity[0.6]],Directive[branchcolor,Thickness[0.0006]]},
     ColorFunction->Function[{x,y,w},Directive[Join[{origColorScheme[argcolorscheme[[1]],
       Arg[f[x,y]]/(2*Pi),argcolorscheme[[2]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {#3&,Function[{x,y,w},Abs[Arg[f[x,y]]]],
       Function[{x,y,w},Im[f[x,y]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
       {Table[{ArcSinh[origMeshF[absC1,N[absC2],i]],
        absContoursCol[contourcolor[[1]],ArcSinh[plotrange],
        ArcSinh[origMeshF[absC1,absC2,i]]]},
        {i,absC3,maxAbsContour}],
       Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
       {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     PlotRange->{ArcSinh[plotrange[[1]]],ArcSinh[plotrange[[2]]]},
     Ticks->{If[Head[ticksXY]===List,ticksXY[[1]],ticksXY],
       If[Head[ticksXY]===List,ticksXY[[2]],ticksXY],
       Join[{{0,0,{0,0}}},
       Table[{N[ArcSinh[10^t]],N[10^t],{0.005,0}},
       {t,ticksexpstep,Floor[Log[10,plotrange[[2]]]],ticksexpstep}]]},
     RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
     PlotRegion->{{0.01,0.99},{0.01,0.99}},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]]]



(************************** FundamentalRegionPlot **************************)

Options[FundamentalRegionPlot]=
  Join[{RegionColor->Automatic, 
        BranchCutColor->Automatic, 
        AbsContours->Automatic, 
        ArgContours->Automatic, 
        ContoursColor->Automatic, 
        ContoursThickness->Automatic, 
        ContoursColorAbsMax->Automatic, 
        ExclusionsType->Automatic,
        DefinitionDomain->Automatic},Options[DensityPlot],Options[ContourPlot]];

FundamentalRegionPlot[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,regioncolor,branchcolor,abscontour,argcontour,contourcolor,
          contourthick,cColorAbsMax,excluType,defindomain,absC1,absC2,absC3,argC1},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    regioncolor=OptionValue[RegionColor];
    branchcolor=OptionValue[BranchCutColor];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    cColorAbsMax=OptionValue[ContoursColorAbsMax];
    excluType=OptionValue[ExclusionsType];
    defindomain=OptionValue[DefinitionDomain];
     If[regioncolor===Automatic,regioncolor={{Hue[0],Hue[0.1]},{Hue[0.5],Hue[0.6]}}];
     If[branchcolor===Automatic,branchcolor=GrayLevel[0]];
     If[abscontour===Automatic,abscontour={"Log",2,{-5,30}}];
     If[argcontour===Automatic,argcontour=8];
     If[contourcolor===Automatic,
       contourcolor={{GrayLevel[0,0.8],GrayLevel[1,0.8]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.001],Thickness[0.001]}];
     If[cColorAbsMax===Automatic,cColorAbsMax=20];
     If[excluType===Automatic,excluType=Automatic];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,{-5,30}},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
   Show[plot3Take[{abscontour,argcontour},
   {DensityPlot[Arg[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
      PlotRange->{-Pi,Pi},AspectRatio->Automatic,Exclusions->None,
      ColorFunction->(If[#>0,Blend[regioncolor[[1]],Abs[#]/Pi],
        Blend[regioncolor[[2]],1-Abs[#]/Pi]]&),
      ColorFunctionScaling->False,
      RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
      Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[DensityPlot]]]]],
    ContourPlot[Abs[Arg[f[x,y]]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
      PlotRange->All,AspectRatio->Automatic,Exclusions->None,ContourShading->None,
      Contours->Table[{j*Pi/argC1,Directive[contourcolor[[2]],contourthick[[2]]]},
        {j,1,argC1-1}],
      RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
      Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[ContourPlot]]]]],
    ContourPlot[Abs[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
      PlotRange->All,AspectRatio->Automatic,
      Exclusions->If[Or[excluType===Automatic,excluType===None,excluType===True],
       excluType,excluType/.z->(x+I y)],
      ExclusionsStyle->{branchcolor,None},
      ContourShading->None,
      Contours->Table[{origMeshF[absC1,N[absC2],i],
        Directive[absContoursCol[contourcolor[[1]],cColorAbsMax,
        origMeshF[absC1,absC2,i]],contourthick[[1]]]},
       {i,absC3[[1]],absC3[[2]]}],
      RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
      Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[ContourPlot]]]]]}],
    PlotRegion->{{0.01,0.98},{0.01,0.97}}]]



(************************* FundamentalRegionPlot3D *************************)

Options[FundamentalRegionPlot3D]=
  Join[{RegionColor->Automatic, 
        BranchCutColor->Automatic, 
        AbsContours->Automatic, 
        ArgContours->Automatic, 
        ContoursColor->Automatic, 
        ContoursThickness->Automatic, 
        ExclusionsType->Automatic, 
        SurfaceProperty->Automatic,
        DefinitionDomain->Automatic},Options[Plot3D]];

FundamentalRegionPlot3D[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,plotrange,boundstyle,regioncolor,branchcolor,abscontour,argcontour,
          contourcolor,contourthick,excluType,surfaceprop,defindomain,absC1,absC2,absC3,
          argC1,maxAbsContour},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    plotrange=OptionValue[PlotRange];
    boundstyle=OptionValue[BoundaryStyle];
    regioncolor=OptionValue[RegionColor];
    branchcolor=OptionValue[BranchCutColor];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    excluType=OptionValue[ExclusionsType];
    surfaceprop=OptionValue[SurfaceProperty];
    defindomain=OptionValue[DefinitionDomain];
     If[regioncolor===Automatic,regioncolor={{Hue[0],Hue[0.1]},{Hue[0.5],Hue[0.6]}}];
     If[branchcolor===Automatic,branchcolor=GrayLevel[0]];
     If[abscontour===Automatic,abscontour={"Log",2,-5}];
     If[argcontour===Automatic,argcontour=8];
     If[contourcolor===Automatic,
        contourcolor={{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[excluType===Automatic,excluType=Automatic];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
     If[boundstyle===GrayLevel[0],boundstyle=Directive[GrayLevel[0],Thickness[0.0005]]];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,-5},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    maxAbsContour=Ceiling[origMeshInvF[absC1,absC2,plotrange[[2]]]];
   Plot3D[If[Abs[f[x,y]]>(1-10^-12)*plotrange[[2]],(1-10^-12)*plotrange[[2]],Abs[f[x,y]]],
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     BoundaryStyle->boundstyle,
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     Exclusions->If[Or[excluType===Automatic,excluType===None,excluType===True],
       excluType,excluType/.z->(x+I y)],
     ExclusionsStyle->Directive[branchcolor,Thickness[0.0015]],
     ColorFunction->Function[{x,y,w},Directive[Join[{If[Arg[f[x,y]]>0,
       Blend[regioncolor[[1]],Abs[Arg[f[x,y]]]/Pi],
       Blend[regioncolor[[2]],1-Abs[Arg[f[x,y]]]/Pi]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {#3&,Function[{x,y,w},Abs[Arg[f[x,y]]]],
       Function[{x,y,w},Im[f[x,y]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
        {Table[{origMeshF[absC1,N[absC2],i],
        absContoursCol[contourcolor[[1]],plotrange,
        origMeshF[absC1,absC2,i]]},
        {i,absC3,maxAbsContour}],
       Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
       {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
     PlotRegion->{{0.01,0.99},{0.01,0.99}},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]]]



(******************************** RePlot3D *********************************)

Options[RePlot3D]=
  Join[{RegionColor->Automatic,
        BranchCutColor->Automatic,
        ReContours->Automatic,
        ImContours->Automatic,
        ContoursColor->Automatic,
        ContoursThickness->Automatic,
        ExclusionsType->Automatic,
        ClippingPlane->Automatic,
        SurfaceProperty->Automatic,
        DefinitionDomain->Automatic},Options[Plot3D]];

RePlot3D[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,plotrange,boundstyle,regioncolor,branchcolor,recontour,imcontour,
          contourcolor,contourthick,excluType,clipPlane,surfaceprop,defindomain,
          reC1,reC2,imC1,imC2,imC3,minReContour,maxReContour},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    plotrange=OptionValue[PlotRange];
    boundstyle=OptionValue[BoundaryStyle];
    regioncolor=OptionValue[RegionColor];
    branchcolor=OptionValue[BranchCutColor];
    recontour=OptionValue[ReContours];
    imcontour=OptionValue[ImContours];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    excluType=OptionValue[ExclusionsType];
    clipPlane=OptionValue[ClippingPlane];
    surfaceprop=OptionValue[SurfaceProperty];
    defindomain=OptionValue[DefinitionDomain];
     If[regioncolor===Automatic,
       regioncolor={{Hue[0],Hue[0,1,1,0.6]},{Hue[0.5],Hue[0.5,1,1,0.6]}}];
     If[branchcolor===Automatic,branchcolor=GrayLevel[0]];
     If[recontour===Automatic,recontour={"Linear",{1,1}}];
     If[imcontour===Automatic,
       imcontour={"Logarithm",{2,2},Floor[arcOMReIm["Logarithm",{2,2},5*Max[Abs[plotrange]]]]}];
     If[contourcolor===Automatic,contourcolor={{Hue[0,1,0.8,1],Hue[0.5,1,0.8,1]},
       {Hue[0.25,1,0.8,0.5],Hue[0.75,1,0.8,0.5]}}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[excluType===Automatic,excluType=Automatic];
     If[clipPlane===Automatic,clipPlane=None,
       If[clipPlane===True,clipPlane={regioncolor[[2,2]],regioncolor[[1,2]]}]];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
     If[boundstyle===GrayLevel[0],boundstyle=Directive[GrayLevel[0],Thickness[0.0005]]];
    {reC1,reC2}=If[MemberQ[{False,None},recontour],{"Linear",{1,1}},recontour];
    {imC1,imC2,imC3}=If[MemberQ[{False,None},imcontour],{"Logarithm",{2,2},1},imcontour];
    minReContour=Ceiling[arcOMReIm[reC1,reC2,plotrange[[1]]]];
    maxReContour=Floor[arcOMReIm[reC1,reC2,plotrange[[2]]]];
   Plot3D[Re[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     Exclusions->If[Or[excluType===Automatic,excluType===None,excluType===True],
       excluType,excluType/.z->(x+I y)],
     ExclusionsStyle->
       {Directive[branchcolor,Opacity[0.6]],Directive[branchcolor,Thickness[0.0006]]},     
     BoundaryStyle->boundstyle,ClippingStyle->clipPlane,
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     ColorFunction->Function[{x,y,w},Directive[Join[{If[w>0,
       Blend[regioncolor[[1]],w/Max[Abs[plotrange]]],
       Blend[regioncolor[[2]],-w/Max[Abs[plotrange]]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,
     MeshFunctions->meshOption2Take[{recontour,imcontour},
       {#3&,Function[{x,y,w},Im[f[x,y]]]}],
     Mesh->meshOption2Take[{recontour,imcontour},
       {Table[{origMeshReIm[reC1,N[reC2],i],
        If[i>=0,contourcolor[[1,1]],contourcolor[[1,2]]]},
        {i,minReContour,maxReContour,1}],
       Table[{origMeshReIm[imC1,N[imC2],j],
        If[j>=0,contourcolor[[2,1]],contourcolor[[2,2]]]},
        {j,-imC3,imC3,1}]}],
     MeshStyle->meshOption2Take[{recontour,imcontour},
       {contourthick[[1]],contourthick[[2]]}],
     RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
     PlotRegion->{{0.01,0.99},{0.01,0.99}},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]]]



(******************************** ImPlot3D *********************************)

Options[ImPlot3D]=
  Join[{RegionColor->Automatic,
        BranchCutColor->Automatic,
        ReContours->Automatic,
        ImContours->Automatic,
        ContoursColor->Automatic,
        ContoursThickness->Automatic,
        ExclusionsType->Automatic,
        ClippingPlane->Automatic,
        SurfaceProperty->Automatic,
        DefinitionDomain->Automatic},Options[Plot3D]];

ImPlot3D[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,plotrange,boundstyle,regioncolor,branchcolor,imcontour,recontour,
          contourcolor,contourthick,excluType,clipPlane,surfaceprop,defindomain,
          imC1,imC2,reC1,reC2,reC3,minImContour,maxImContour},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    plotrange=OptionValue[PlotRange];
    boundstyle=OptionValue[BoundaryStyle];
    regioncolor=OptionValue[RegionColor];
    branchcolor=OptionValue[BranchCutColor];
    imcontour=OptionValue[ImContours];
    recontour=OptionValue[ReContours];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    excluType=OptionValue[ExclusionsType];
    clipPlane=OptionValue[ClippingPlane];
    surfaceprop=OptionValue[SurfaceProperty];
    defindomain=OptionValue[DefinitionDomain];
     If[regioncolor===Automatic,
       regioncolor={{Hue[0.25],Hue[0.25,1,1,0.6]},{Hue[0.75],Hue[0.75,1,1,0.6]}}];
     If[branchcolor===Automatic,branchcolor=GrayLevel[0]];
     If[imcontour===Automatic,imcontour={"Linear",{1,1}}];
     If[recontour===Automatic,
       recontour={"Logarithm",{2,2},Floor[arcOMReIm["Logarithm",{2,2},5*Max[Abs[plotrange]]]]}];
     If[contourcolor===Automatic,contourcolor={{Hue[0.25,1,0.8,1],Hue[0.75,1,0.8,1]},
       {Hue[0,1,0.8,0.5],Hue[0.5,1,0.8,0.5]}}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[excluType===Automatic,excluType=Automatic];
     If[clipPlane===Automatic,clipPlane=None,
       If[clipPlane===True,clipPlane={regioncolor[[2,2]],regioncolor[[1,2]]}]];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
     If[boundstyle===GrayLevel[0],boundstyle=Directive[GrayLevel[0],Thickness[0.0005]]];
    {imC1,imC2}=If[MemberQ[{False,None},imcontour],{"Linear",{1,1}},imcontour];
    {reC1,reC2,reC3}=If[MemberQ[{False,None},recontour],{"Logarithm",{2,2},1},recontour];
    minImContour=Ceiling[arcOMReIm[imC1,imC2,plotrange[[1]]]];
    maxImContour=Floor[arcOMReIm[imC1,imC2,plotrange[[2]]]];
   Plot3D[Im[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     Exclusions->If[Or[excluType===Automatic,excluType===None,excluType===True],
       excluType,excluType/.z->(x+I y)],
     ExclusionsStyle->
       {Directive[branchcolor,Opacity[0.6]],Directive[branchcolor,Thickness[0.0006]]},     
     BoundaryStyle->boundstyle,ClippingStyle->clipPlane,
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     ColorFunction->Function[{x,y,w},Directive[Join[{If[w>0,
       Blend[regioncolor[[1]],w/Max[Abs[plotrange]]],
       Blend[regioncolor[[2]],-w/Max[Abs[plotrange]]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,
     MeshFunctions->meshOption2Take[{imcontour,recontour},
       {#3&,Function[{x,y,w},Re[f[x,y]]]}],
     Mesh->meshOption2Take[{imcontour,recontour},
       {Table[{origMeshReIm[imC1,N[imC2],i],
        If[i>=0,contourcolor[[1,1]],contourcolor[[1,2]]]},
        {i,minImContour,maxImContour,1}],
       Table[{origMeshReIm[reC1,N[reC2],j],
        If[j>=0,contourcolor[[2,1]],contourcolor[[2,2]]]},
        {j,-reC3,reC3,1}]}],
     MeshStyle->meshOption2Take[{imcontour,recontour},
       {contourthick[[1]],contourthick[[2]]}],
     RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
     PlotRegion->{{0.01,0.99},{0.01,0.99}},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]]]



(**************************** TopographyPlot3D *****************************)

Options[TopographyPlot3D]=
  Join[{AbsContours->Automatic,
        ArgContours->Automatic,
        ContoursColor->Automatic, 
        ContoursThickness->Automatic,
        LandColorScheme->Automatic,
        LandFillingColor->Automatic,
        SeaLevel->Automatic,
        SeaColor->Automatic,
        SeaWave->Automatic, 
        SurfaceProperty->Automatic},Options[Plot3D]];

TopographyPlot3D[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,geopl,seapl,plotrange,abscontour,argcontour,contourcolor,contourthick,
          landcolscheme,landfillcolor,sealevel,seacolor,seawave,surfaceprop,
          absC1,absC2,absC3,argC1,maxAbsContours},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    plotrange=OptionValue[PlotRange];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    landcolscheme=OptionValue[LandColorScheme];
    landfillcolor=OptionValue[LandFillingColor];
    sealevel=OptionValue[SeaLevel];
    seacolor=OptionValue[SeaColor];
    seawave=OptionValue[SeaWave];
    surfaceprop=OptionValue[SurfaceProperty];
     If[abscontour===Automatic,abscontour={"LinearLog",2,-10}];
     If[argcontour===Automatic,argcontour=8];
     If[contourcolor===Automatic,contourcolor={GrayLevel[0,0.4],GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[landcolscheme===Automatic,landcolscheme={"geography",{1,1,1}}];
     If[landfillcolor===Automatic,landfillcolor=Hue[0.1,0.5,0.4]];
     If[sealevel===Automatic,sealevel=1];
     If[seacolor===Automatic,seacolor={Hue[0.6,0.8,1,0.5],Hue[0.6,1,0.8,0.6]}];
     If[seawave===Automatic,seawave={0,{0,0}}];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"LinearLog",2,-10},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    maxAbsContours=Ceiling[origMeshInvF[absC1,absC2,plotrange[[2]]]];
   geopl=Plot3D[Abs[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     ClippingStyle->landColorFunc[landcolscheme[[1]],0.99,landcolscheme[[2]]],
     ExclusionsStyle->{Hue[0.05],Directive[Hue[0.05],Thickness[0.002]]},
     ColorFunction->Function[{x,y,w},Directive[Join[{
       landColorFunc[landcolscheme[[1]],1-2^-w,landcolscheme[[2]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,
     BoundaryStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {#3&,Function[{x,y,w},Abs[Arg[f[x,y]]]],
       Function[{x,y,w},Im[f[x,y]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
       {Table[{origMeshF[absC1,N[absC2],i],contourcolor[[1]]},
        {i,absC3,maxAbsContours}],
       Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
       {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     Filling->Bottom,FillingStyle->landfillcolor,
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]];
   seapl=Plot3D[sealevel+seawave[[1]]*plotrange[[2]]/100*
       Sin[2*Pi*seawave[[2,1]]/Re[z1-z0]*x+BesselJ[0,2 Pi*seawave[[2,2]]/Im[z1-z0] y]],
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     Mesh->False,PlotStyle->seacolor[[1]],Filling->Bottom,
     FillingStyle->seacolor[[2]],BoundaryStyle->None,
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     RegionFunction->Function[{x,y,w},(Re[z0]<x/0.999<Re[z1] && Im[z0]<y/0.999<Im[z1])],
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]];
   Show[geopl,seapl,PlotRegion->{{0.01,0.99},{0.01,0.99}}]]



(*************************** RiemannSpherePlot3D ***************************)

Options[RiemannSpherePlot3D]=
  Join[{ZeroPointAltitude->Automatic,
        FunctionMaxRange->Automatic,
        RiemannSphereType->Automatic,
        AbsContours->Automatic,
        ArgContours->Automatic,
        ArgColorScheme->Automatic,
        ContoursColor->Automatic,
        ContoursThickness->Automatic,
        AdjointSphere->Automatic,
        AdjointMesh->Automatic,
        AdjointMeshOpacity->Automatic,
        AdjointSphereOpacity->Automatic,
        AdjointArrowLength->Automatic,
        SurfaceProperty->Automatic},Options[SphericalPlot3D]];

RiemannSpherePlot3D[unknownFunc_,{z_,r0_,{ph0_,ph1_}},OptionsPattern[]]:=
  Module[{f,x,y,w,theta,phi,r,funcP,spheP,grPrim,zeroalt,fmaxrange,rstype,abscontour,
          argcontour,argcolorscheme,contourcolor,contourthick,adjsph,adjmesh,
          adjmeshopac,adjsphopac,adjarlen,surfaceprop,absC1,absC2,absC3,argC1},
    zeroalt=OptionValue[ZeroPointAltitude];
    fmaxrange=OptionValue[FunctionMaxRange];
    rstype=OptionValue[RiemannSphereType];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    adjsph=OptionValue[AdjointSphere];
    adjmesh=OptionValue[AdjointMesh];
    adjmeshopac=OptionValue[AdjointMeshOpacity];
    adjsphopac=OptionValue[AdjointSphereOpacity];
    adjarlen=OptionValue[AdjointArrowLength];
    surfaceprop=OptionValue[SurfaceProperty];
     If[zeroalt===Automatic,zeroalt=10];
     If[fmaxrange===Automatic,fmaxrange=10];
     If[rstype===Automatic,rstype=1];
     If[abscontour===Automatic,abscontour={"Log",2,-5}];
     If[argcontour===Automatic,argcontour=8];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,1}}];
     If[contourcolor===Automatic,
       contourcolor={{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[adjsph===Automatic,adjsph=True];
     If[adjmesh===Automatic,adjmesh={6,4}];
     If[adjmeshopac===Automatic,adjmeshopac=1];
     If[adjsphopac===Automatic,adjsphopac=0.1];
     If[adjarlen===Automatic,adjarlen={0.3,0.3}];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,-5},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    f[theta_,phi_]:=unknownFunc/.z->(rsTypeCot[theta/2,rstype]*Exp[phi*I]);
   funcP=SphericalPlot3D[zeroalt+If[Abs[f[theta,phi]]>fmaxrange,
     fmaxrange,Abs[f[theta,phi]]],
     {theta,rsTypeThetaMin[r0,rstype],rsTypeThetaMax[r0,rstype]},
     {phi,ph0,ph1+0.0000001},
     ColorFunction->Function[{x,y,w,theta,phi,r},
       Directive[Join[{origColorScheme[argcolorscheme[[1]],
       Arg[f[theta,phi]]/(2*Pi),argcolorscheme[[2]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,BoundaryStyle->None,
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {Function[{x,y,w,theta,phi,r},Abs[f[theta,phi]]],
        Function[{x,y,w,theta,phi,r},Abs[Arg[f[theta,phi]]]],
        Function[{x,y,w,theta,phi,r},Im[f[theta,phi]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
       {Table[{origMeshF[absC1,N[absC2],i],
        absContoursCol[contourcolor[[1]],fmaxrange,
        origMeshF[absC1,absC2,i]]},
        {i,absC3,Ceiling[origMeshInvF[absC1,absC2,fmaxrange]]}],
       Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
       {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[SphericalPlot3D]]]]];
   spheP=If[adjsph===False,None,
    SphericalPlot3D[1.0001*(zeroalt+fmaxrange),{theta,0,Pi},{phi,0,2*Pi+0.0000001},
     MeshFunctions->{Function[{x,y,w,theta,phi,r},Cot[theta/2]],
       Function[{x,y,w,theta,phi,r},phi]},
     Mesh->{Join[Table[{If[rstype===1,i,1/i],GrayLevel[(i-1)/(adjmesh[[1]]-1)/2+1/2,adjmeshopac]},
       {i,1,adjmesh[[1]]}],
       Table[{If[rstype===1,1/j,j],GrayLevel[(adjmesh[[1]]-j)/(adjmesh[[1]]-1)/2,adjmeshopac]},
       {j,1,adjmesh[[1]]}]],
       Table[{k*Pi/adjmesh[[2]],Hue[k/(2*adjmesh[[2]]),1,1,adjmeshopac]},
       {k,1,2*adjmesh[[2]]}]},
     ColorFunction->Function[{x,y,w,theta,phi,r},
       Hue[phi/(2 Pi),If[theta/Pi<0.5,2*theta/Pi,1],
       If[theta/Pi<0.5,1,2-2*theta/Pi],adjsphopac]],
     ColorFunctionScaling->False,BoundaryStyle->None,
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[SphericalPlot3D]]]]]];
   grPrim=Graphics3D[{
     Arrowheads[0.03],
     Arrow[{{0,0,-If[rstype===1,1+adjarlen[[1]],1+adjarlen[[2]]]*(zeroalt+fmaxrange)},
       {0,0,-(zeroalt+fmaxrange)}}],
     Arrow[{{0,0,If[rstype===1,1+adjarlen[[2]],1+adjarlen[[1]]]*(zeroalt+fmaxrange)},
       {0,0,(zeroalt+fmaxrange)}}],
     Text[0,
       {0,0,1.07*If[rstype===1,-1-adjarlen[[1]],1+adjarlen[[1]]]*(zeroalt+fmaxrange)}],
     Text[Infinity,
       {0,0,1.07*If[rstype===1,1+adjarlen[[2]],-1-adjarlen[[2]]]*(zeroalt+fmaxrange)}]}];
   Show[If[spheP===None,funcP,{funcP,spheP,grPrim}],
     PlotRange->All,Boxed->False,Axes->False,PlotRegion->{{0.01,0.99},{0.01,0.99}}]]



(************************ RiemannSphereDensityPlot *************************)

Options[RiemannSphereDensityPlot]=
  Join[{RiemannSphereType->Automatic,
        AbsContours->Automatic,
        ArgContours->Automatic,
        ArgColorScheme->Automatic,
        ContoursColor->Automatic,
        ContoursThickness->Automatic,
        AdjointSphere->Automatic,
        AdjointSphereRadius->Automatic,
        AdjointMesh->Automatic,
        AdjointMeshOpacity->Automatic,
        AdjointSphereOpacity->Automatic,
        AdjointArrowLength->Automatic,
        ContoursColorAbsMax->Automatic,
        SurfaceProperty->Automatic},Options[SphericalPlot3D]];

RiemannSphereDensityPlot[unknownFunc_,{z_,r0_,{ph0_,ph1_}},OptionsPattern[]]:=
  Module[{f,x,y,w,theta,phi,r,funcP,spheP,grPrim,rstype,abscontour,argcontour,argcolorscheme,
          contourcolor,contourthick,adjsph,adjsphrad,adjmesh,adjmeshopac,adjsphopac,
          adjarlen,cColorAbsMax,surfaceprop,absC1,absC2,absC3,argC1},
    rstype=OptionValue[RiemannSphereType];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    adjsph=OptionValue[AdjointSphere];
    adjsphrad=OptionValue[AdjointSphereRadius];
    adjmesh=OptionValue[AdjointMesh];
    adjmeshopac=OptionValue[AdjointMeshOpacity];
    adjsphopac=OptionValue[AdjointSphereOpacity];
    adjarlen=OptionValue[AdjointArrowLength];
    cColorAbsMax=OptionValue[ContoursColorAbsMax];
    surfaceprop=OptionValue[SurfaceProperty];
     If[rstype===Automatic,rstype=1];
     If[abscontour===Automatic,abscontour={"Log",2,{-5,30}}];
     If[argcontour===Automatic,argcontour=8];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,1}}];
     If[contourcolor===Automatic,
       contourcolor={{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[adjsph===Automatic,adjsph=True];
     If[adjsphrad===Automatic,adjsphrad=1.2];
     If[adjmesh===Automatic,adjmesh={6,4}];
     If[adjmeshopac===Automatic,adjmeshopac=0.7];
     If[adjsphopac===Automatic,adjsphopac=0.1];
     If[adjarlen===Automatic,adjarlen={0.3,0.3}];
     If[cColorAbsMax===Automatic,cColorAbsMax=20];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,{-5,30}},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    f[theta_,phi_]:=unknownFunc/.z->(rsTypeCot[theta/2,rstype]*Exp[phi*I]);
   funcP=SphericalPlot3D[1,
     {theta,rsTypeThetaMin[r0,rstype],rsTypeThetaMax[r0,rstype]},
     {phi,ph0,ph1+0.0000001},
     ColorFunction->Function[{x,y,w,theta,phi,r},
       Directive[Join[{origColorScheme[argcolorscheme[[1]],
       Arg[f[theta,phi]]/(2*Pi),argcolorscheme[[2]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,BoundaryStyle->None,
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {Function[{x,y,w,theta,phi,r},Abs[f[theta,phi]]],
        Function[{x,y,w,theta,phi,r},Abs[Arg[f[theta,phi]]]],
        Function[{x,y,w,theta,phi,r},Im[f[theta,phi]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
       {Table[{origMeshF[absC1,N[absC2],i],
        absContoursCol[contourcolor[[1]],cColorAbsMax,
        origMeshF[absC1,absC2,i]]},
        {i,absC3[[1]],absC3[[2]]}],
       Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
       {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[SphericalPlot3D]]]]];
   spheP=If[adjsph===False,None,
    SphericalPlot3D[adjsphrad,{theta,0,Pi},{phi,0,2*Pi+0.0000001},
     MeshFunctions->{Cot[#4/2]&,#5&},
     Mesh->{Join[Table[{If[rstype===1,i,1/i],GrayLevel[(i-1)/(adjmesh[[1]]-1)/2+1/2,adjmeshopac]},
       {i,1,adjmesh[[1]]}],
       Table[{If[rstype===1,1/j,j],GrayLevel[(adjmesh[[1]]-j)/(adjmesh[[1]]-1)/2,adjmeshopac]},
       {j,1,adjmesh[[1]]}]],
       Table[{k*Pi/adjmesh[[2]],Hue[k/(2*adjmesh[[2]]),1,1,adjmeshopac]},
       {k,1,2*adjmesh[[2]]}]},
     ColorFunction->(Hue[#5/(2 Pi),If[#4/Pi<0.5,2*#4/Pi,1],
       If[#4/Pi<0.5,1,2-2*#4/Pi],adjsphopac]&),
     ColorFunctionScaling->False,BoundaryStyle->None,
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[SphericalPlot3D]]]]]];
   grPrim=Graphics3D[{
     Arrowheads[0.03],
     Arrow[{{0,0,-If[rstype===1,1+adjarlen[[1]],1+adjarlen[[2]]]*adjsphrad},
       {0,0,-adjsphrad}}],
     Arrow[{{0,0,If[rstype===1,1+adjarlen[[2]],1+adjarlen[[1]]]*adjsphrad},
       {0,0,adjsphrad}}],
     Text[0,
       {0,0,1.07*If[rstype===1,-1-adjarlen[[1]],1+adjarlen[[1]]]*adjsphrad}],
     Text[Infinity,
       {0,0,1.07*If[rstype===1,1+adjarlen[[2]],-1-adjarlen[[2]]]*adjsphrad}]}];
   Show[If[spheP===None,funcP,{funcP,spheP,grPrim}],
     PlotRange->All,Boxed->False,Axes->False,PlotRegion->{{0.01,0.99},{0.01,0.99}}]]



(************************* RiemannSphereLogPlot3D **************************)

Options[RiemannSphereLogPlot3D]=
  Join[{ZeroPointAltitude->Automatic,
        FunctionMaxRange->Automatic,
        RiemannSphereType->Automatic,
        AbsContours->Automatic,
        ArgContours->Automatic,
        ArgColorScheme->Automatic,
        ContoursColor->Automatic,
        ContoursThickness->Automatic,
        AdjointSphere->Automatic,
        AdjointMesh->Automatic,
        AdjointMeshOpacity->Automatic,
        AdjointSphereOpacity->Automatic,
        AdjointArrowLength->Automatic,
        SurfaceProperty->Automatic},Options[SphericalPlot3D]];

RiemannSphereLogPlot3D[unknownFunc_,{z_,r0_,{ph0_,ph1_}},OptionsPattern[]]:=
  Module[{f,x,y,w,theta,phi,r,funcP,spheP,grPrim,zeroalt,fmaxrange,rstype,abscontour,argcontour,
          argcolorscheme,contourcolor,contourthick,adjsph,adjmesh,adjmeshopac,adjsphopac,
          adjarlen,surfaceprop,absC1,absC2,absC3,argC1},
    zeroalt=OptionValue[ZeroPointAltitude];
    fmaxrange=OptionValue[FunctionMaxRange];
    rstype=OptionValue[RiemannSphereType];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    adjsph=OptionValue[AdjointSphere];
    adjmesh=OptionValue[AdjointMesh];
    adjmeshopac=OptionValue[AdjointMeshOpacity];
    adjsphopac=OptionValue[AdjointSphereOpacity];
    adjarlen=OptionValue[AdjointArrowLength];
    surfaceprop=OptionValue[SurfaceProperty];
     If[zeroalt===Automatic,zeroalt=10];
     If[fmaxrange===Automatic,fmaxrange=10000];
     If[rstype===Automatic,rstype=1];
     If[abscontour===Automatic,abscontour={"Log",2,-5}];
     If[argcontour===Automatic,argcontour=8];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,1}}];
     If[contourcolor===Automatic,
       contourcolor={{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[adjsph===Automatic,adjsph=True];
     If[adjmesh===Automatic,adjmesh={6,4}];
     If[adjmeshopac===Automatic,adjmeshopac=1];
     If[adjsphopac===Automatic,adjsphopac=0.1];
     If[adjarlen===Automatic,adjarlen={0.3,0.3}];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,-5},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    f[theta_,phi_]:=unknownFunc/.z->(rsTypeCot[theta/2,rstype]*Exp[phi*I]);
   funcP=SphericalPlot3D[zeroalt+If[Log[10,Abs[f[theta,phi]]]>Log[10,fmaxrange],
     Log[10,fmaxrange],Log[10,Abs[f[theta,phi]]]],
     {theta,rsTypeThetaMin[r0,rstype],rsTypeThetaMax[r0,rstype]},
     {phi,ph0,ph1+0.0000001},
     ColorFunction->Function[{x,y,w,theta,phi,r},
       Directive[Join[{origColorScheme[argcolorscheme[[1]],
       Arg[f[theta,phi]]/(2*Pi),argcolorscheme[[2]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,BoundaryStyle->None,
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {Function[{x,y,w,theta,phi,r},Log[10,Abs[f[theta,phi]]]],
        Function[{x,y,w,theta,phi,r},Abs[Arg[f[theta,phi]]]],
        Function[{x,y,w,theta,phi,r},Im[f[theta,phi]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
       {Table[{Log[10,origMeshF[absC1,N[absC2],i]],
        absContoursCol[contourcolor[[1]],Log[10,fmaxrange],
        Log[10,origMeshF[absC1,absC2,i]]]},
        {i,absC3,Ceiling[origMeshInvF[absC1,absC2,fmaxrange]]}],
       Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
       {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[SphericalPlot3D]]]]];
   spheP=If[adjsph===False,None,
    SphericalPlot3D[1.0001*(zeroalt+Log[10,fmaxrange]),
     {theta,0,Pi},{phi,0,2*Pi+0.0000001},
     MeshFunctions->{Cot[#4/2]&,#5&},
     Mesh->{Join[Table[{If[rstype===1,i,1/i],GrayLevel[(i-1)/(adjmesh[[1]]-1)/2+1/2,adjmeshopac]},
       {i,1,adjmesh[[1]]}],
       Table[{If[rstype===1,1/j,j],GrayLevel[(adjmesh[[1]]-j)/(adjmesh[[1]]-1)/2,adjmeshopac]},
       {j,1,adjmesh[[1]]}]],
       Table[{k*Pi/adjmesh[[2]],Hue[k/(2*adjmesh[[2]]),1,1,adjmeshopac]},
       {k,1,2*adjmesh[[2]]}]},
     ColorFunction->(Hue[#5/(2 Pi),If[#4/Pi<0.5,2*#4/Pi,1],
       If[#4/Pi<0.5,1,2-2*#4/Pi],adjsphopac]&),
     ColorFunctionScaling->False,BoundaryStyle->None,
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[SphericalPlot3D]]]]]];
   grPrim=Graphics3D[{
     Arrowheads[0.03],
     Arrow[{{0,0,-If[rstype===1,1+adjarlen[[1]],1+adjarlen[[2]]]*(zeroalt+Log[10,fmaxrange])},
       {0,0,-(zeroalt+Log[10,fmaxrange])}}],
     Arrow[{{0,0,If[rstype===1,1+adjarlen[[2]],1+adjarlen[[1]]]*(zeroalt+Log[10,fmaxrange])},
       {0,0,(zeroalt+Log[10,fmaxrange])}}],
     Text[0,
       {0,0,1.07*If[rstype===1,-1-adjarlen[[1]],1+adjarlen[[1]]]*(zeroalt+Log[10,fmaxrange])}],
     Text[Infinity,
       {0,0,1.07*If[rstype===1,1+adjarlen[[2]],-1-adjarlen[[2]]]*(zeroalt+Log[10,fmaxrange])}]}];
   Show[If[spheP===None,funcP,{funcP,spheP,grPrim}],
     PlotRange->All,Boxed->False,Axes->False,PlotRegion->{{0.01,0.99},{0.01,0.99}}]]



(*********************** RiemannSphereArcSinhPlot3D ************************)

Options[RiemannSphereArcSinhPlot3D]=
  Join[{ZeroPointAltitude->Automatic,
        FunctionMaxRange->Automatic,
        RiemannSphereType->Automatic,
        AbsContours->Automatic,
        ArgContours->Automatic,
        ArgColorScheme->Automatic,
        ContoursColor->Automatic,
        ContoursThickness->Automatic,
        AdjointSphere->Automatic,
        AdjointMesh->Automatic,
        AdjointMeshOpacity->Automatic,
        AdjointSphereOpacity->Automatic,
        AdjointArrowLength->Automatic,
        SurfaceProperty->Automatic},Options[SphericalPlot3D]];

RiemannSphereArcSinhPlot3D[unknownFunc_,{z_,r0_,{ph0_,ph1_}},OptionsPattern[]]:=
  Module[{f,x,y,w,theta,phi,r,funcP,spheP,grPrim,zeroalt,fmaxrange,rstype,abscontour,argcontour,
          argcolorscheme,contourcolor,contourthick,adjsph,adjmesh,adjmeshopac,adjsphopac,
          adjarlen,surfaceprop,absC1,absC2,absC3,argC1},
    zeroalt=OptionValue[ZeroPointAltitude];
    fmaxrange=OptionValue[FunctionMaxRange];
    rstype=OptionValue[RiemannSphereType];
    abscontour=OptionValue[AbsContours];
    argcontour=OptionValue[ArgContours];
    argcolorscheme=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourcolor=OptionValue[ContoursColor];
    contourthick=OptionValue[ContoursThickness];
    adjsph=OptionValue[AdjointSphere];
    adjmesh=OptionValue[AdjointMesh];
    adjmeshopac=OptionValue[AdjointMeshOpacity];
    adjsphopac=OptionValue[AdjointSphereOpacity];
    adjarlen=OptionValue[AdjointArrowLength];
    surfaceprop=OptionValue[SurfaceProperty];
     If[zeroalt===Automatic,zeroalt=10];
     If[fmaxrange===Automatic,fmaxrange=10000];
     If[rstype===Automatic,rstype=1];
     If[abscontour===Automatic,abscontour={"Log",2,-5}];
     If[argcontour===Automatic,argcontour=8];
     If[argcolorscheme===Automatic,argcolorscheme={"Hue",{1,1,1}}];
     If[contourcolor===Automatic,
       contourcolor={{GrayLevel[0,0.6],GrayLevel[1,0.6]},GrayLevel[0,0.4]}];
     If[contourthick===Automatic,contourthick={Thickness[0.0015],Thickness[0.0015]}];
     If[adjsph===Automatic,adjsph=True];
     If[adjmesh===Automatic,adjmesh={6,4}];
     If[adjmeshopac===Automatic,adjmeshopac=1];
     If[adjsphopac===Automatic,adjsphopac=0.1];
     If[adjarlen===Automatic,adjarlen={0.3,0.3}];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
    {absC1,absC2,absC3}=If[MemberQ[{False,None},abscontour],{"Log",2,-5},abscontour];
    argC1=If[MemberQ[{False,None},argcontour],8,argcontour];
    f[theta_,phi_]=unknownFunc/.z->(rsTypeCot[theta/2,rstype]*Exp[phi*I]);
   funcP=SphericalPlot3D[zeroalt+If[ArcSinh[Abs[f[theta,phi]]]>ArcSinh[fmaxrange],
     ArcSinh[fmaxrange],ArcSinh[Abs[f[theta,phi]]]],
     {theta,rsTypeThetaMin[r0,rstype],rsTypeThetaMax[r0,rstype]},
     {phi,ph0,ph1+0.0000001},
     ColorFunction->Function[{x,y,w,theta,phi,r},
       Directive[Join[{origColorScheme[argcolorscheme[[1]],
       Arg[f[theta,phi]]/(2*Pi),argcolorscheme[[2]]]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,BoundaryStyle->None,
     MeshFunctions->meshOption3Take[{abscontour,argcontour},
       {Function[{x,y,w,theta,phi,r},ArcSinh[Abs[f[theta,phi]]]],
        Function[{x,y,w,theta,phi,r},Abs[Arg[f[theta,phi]]]],
        Function[{x,y,w,theta,phi,r},Im[f[theta,phi]]]}],
     Mesh->meshOption3Take[{abscontour,argcontour},
       {Table[{ArcSinh[origMeshF[absC1,N[absC2],i]],
        absContoursCol[contourcolor[[1]],ArcSinh[fmaxrange],
        ArcSinh[origMeshF[absC1,absC2,i]]]},
        {i,absC3,Ceiling[origMeshInvF[absC1,absC2,fmaxrange]]}],
       Table[{j*Pi/argC1,contourcolor[[2]]},{j,1,argC1-1}],
       {{0,contourcolor[[2]]}}}],
     MeshStyle->meshOption3Take[{abscontour,argcontour},
       {contourthick[[1]],contourthick[[2]],contourthick[[2]]}],
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[SphericalPlot3D]]]]];
   spheP=If[adjsph===False,None,
    SphericalPlot3D[1.0001*(zeroalt+ArcSinh[fmaxrange]),
     {theta,0,Pi},{phi,0,2*Pi+0.0000001},
     MeshFunctions->{Cot[#4/2]&,#5&},
     Mesh->{Join[Table[{If[rstype===1,i,1/i],GrayLevel[(i-1)/(adjmesh[[1]]-1)/2+1/2,adjmeshopac]},
       {i,1,adjmesh[[1]]}],
      Table[{If[rstype===1,1/j,j],GrayLevel[(adjmesh[[1]]-j)/(adjmesh[[1]]-1)/2,adjmeshopac]},
       {j,1,adjmesh[[1]]}]],
      Table[{k*Pi/adjmesh[[2]],Hue[k/(2*adjmesh[[2]]),1,1,adjmeshopac]},
       {k,1,2*adjmesh[[2]]}]},
     ColorFunction->(Hue[#5/(2 Pi),If[#4/Pi<0.5,2*#4/Pi,1],
       If[#4/Pi<0.5,1,2-2*#4/Pi],adjsphopac]&),
     ColorFunctionScaling->False,BoundaryStyle->None,
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[SphericalPlot3D]]]]]];
   grPrim=Graphics3D[{
     Arrowheads[0.03],
     Arrow[{{0,0,-If[rstype===1,1+adjarlen[[1]],1+adjarlen[[2]]]*(zeroalt+ArcSinh[fmaxrange])},
       {0,0,-(zeroalt+ArcSinh[fmaxrange])}}],
     Arrow[{{0,0,If[rstype===1,1+adjarlen[[2]],1+adjarlen[[1]]]*(zeroalt+ArcSinh[fmaxrange])},
       {0,0,(zeroalt+ArcSinh[fmaxrange])}}],
     Text[0,
       {0,0,1.07*If[rstype===1,-1-adjarlen[[1]],1+adjarlen[[1]]]*(zeroalt+ArcSinh[fmaxrange])}],
     Text[Infinity,
       {0,0,1.07*If[rstype===1,1+adjarlen[[2]],-1-adjarlen[[2]]]*(zeroalt+ArcSinh[fmaxrange])}]}];
   Show[If[spheP===None,funcP,{funcP,spheP,grPrim}],
     PlotRange->All,Boxed->False,Axes->False,PlotRegion->{{0.01,0.99},{0.01,0.99}}]]



(*************************** NonMeshAbsArgPlot3D ***************************)

Options[NonMeshAbsArgPlot3D]=
  Join[{AbsContours->Automatic, 
        ArgColorScheme->Automatic, 
        ContoursOpacity->Automatic, 
        SurfaceProperty->Automatic,
        DefinitionDomain->Automatic},Options[Plot3D]];

NonMeshAbsArgPlot3D[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,w,plotrange,boundstyle,abscontour,argcolsch,contourOpa,surfaceprop,defindomain},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    plotrange=OptionValue[PlotRange];
    boundstyle=OptionValue[BoundaryStyle];
    abscontour=OptionValue[AbsContours];
    argcolsch=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourOpa=OptionValue[ContoursOpacity];
    surfaceprop=OptionValue[SurfaceProperty];
    defindomain=OptionValue[DefinitionDomain];
     If[abscontour===Automatic,abscontour={"Log",2}];
     If[argcolsch===Automatic,argcolsch={"Hue",{1,1,1}}];
     If[contourOpa===Automatic,contourOpa=0.4];
     If[surfaceprop===Automatic,surfaceprop=Specularity[GrayLevel[0.5],40]];
     If[boundstyle===GrayLevel[0],boundstyle=Directive[GrayLevel[0],Thickness[0.0005]]];
   Plot3D[Min[Abs[f[x,y]],(1-10^-12)*plotrange[[2]]],
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},Exclusions->None,Mesh->None,
     ColorFunction->Function[{x,y,w},Directive[Join[{origColorScheme[argcolsch[[1]],
       Arg[f[x,y]]/(2*Pi),argcolsch[[2]]*
       {scaleSat[abscontour,plotrange[[2]],Abs[f[x,y]],contourOpa],
        scaleBri[abscontour,plotrange[[2]],Abs[f[x,y]],contourOpa],1}]},
       If[Head[surfaceprop]===Directive,Apply[List,surfaceprop],{surfaceprop}]]]],
     ColorFunctionScaling->False,
     BoundaryStyle->boundstyle,
     BoxStyle->Directive[GrayLevel[0],Thickness[0.0005]],
     RegionFunction->If[defindomain===Automatic,(True&),
        Function[{x,y,w},defindomain/.z->(x+I y)]],
     PlotRegion->{{0.01,0.99},{0.01,0.99}},
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[Plot3D]]]]]]



(************************** NonMeshArgDensityPlot **************************)

Options[NonMeshArgDensityPlot]=
  Join[{AbsContours->Automatic, 
        ArgColorScheme->Automatic, 
        ContoursOpacity->Automatic, 
        ContoursColorAbsMax->Automatic,
        DefinitionDomain->Automatic},Options[DensityPlot]];

NonMeshArgDensityPlot[unknownFunc_,{z_,z0_,z1_},OptionsPattern[]]:=
  Module[{f,x,y,densP1,densP2,abscontour,argcolsch,contourOpa,cColorAbsMax,defindomain},
    f[x_,y_]:=unknownFunc/.z->(x+I y);
    abscontour=OptionValue[AbsContours];
    argcolsch=argColSchOptCases[OptionValue[ArgColorScheme]];
    contourOpa=OptionValue[ContoursOpacity];
    cColorAbsMax=OptionValue[ContoursColorAbsMax];
    defindomain=OptionValue[DefinitionDomain];
     If[abscontour===Automatic,abscontour={"Log",2}];
     If[argcolsch===Automatic,argcolsch={"Hue",{1,1,1}}];
     If[contourOpa===Automatic,contourOpa=0.4];
     If[cColorAbsMax===Automatic,cColorAbsMax=20];
   densP1=DensityPlot[Arg[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     PlotRange->{-Pi,Pi},AspectRatio->Automatic,Exclusions->None,
     ColorFunction->(origColorScheme[argcolsch[[1]],#/(2*Pi),argcolsch[[2]]]&),
     ColorFunctionScaling->False,
     RegionFunction->If[defindomain===Automatic,(True&),
       Function[{x,y,w},defindomain/.z->(x+I y)]],
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[DensityPlot]]]]];
   densP2=DensityPlot[Abs[f[x,y]],{x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     PlotRange->All,AspectRatio->Automatic,Exclusions->None,
     ColorFunction->(GrayLevel[colorLevelFP[
       origMeshInvF[abscontour[[1]],abscontour[[2]],cColorAbsMax],
       origMeshInvF[abscontour[[1]],abscontour[[2]],#]]*argcolsch[[2,2]],
       opacityLevelF[abscontour,#,contourOpa]*argcolsch[[2,3]]]&),
     ColorFunctionScaling->False,
     RegionFunction->If[defindomain===Automatic,(True&),
       Function[{x,y,w},defindomain/.z->(x+I y)]],
     Evaluate[Map[(#->OptionValue[#])&,Map[#[[1]]&,Options[DensityPlot]]]]];
   Show[densP1,densP2,PlotRegion->{{0.01,0.98},{0.01,0.97}}]]



(*********************************** End ***********************************)

End[]

Protect[
  GridLinesPlot, GridLinesParametricPlot, GridLinesColor, FundamentalRectangle, 
  AbsArgPlot, ArgColorScheme, ReImPlot, ReImFillingColors, ZeroPointsStyle, 
  AbsArgPlot3D, ArgDensityPlot, AbsContours, ArgContours, ContoursColor, 
  ContoursThickness, ContoursColorAbsMax, SurfaceProperty, DefinitionDomain, 
  AbsArgLogPlot3D, AbsArgArcSinhPlot3D, TicksExpStep, 
  FundamentalRegionPlot, FundamentalRegionPlot3D, RegionColor, BranchCutColor, 
  RePlot3D, ImPlot3D, ReContours, ImContours, ExclusionsType, ClippingPlane, 
  TopographyPlot3D, LandColorScheme, LandFillingColor, SeaLevel, SeaColor, SeaWave, 
  RiemannSpherePlot3D, RiemannSphereDensityPlot, RiemannSphereLogPlot3D, 
  RiemannSphereArcSinhPlot3D, ZeroPointAltitude, FunctionMaxRange, RiemannSphereType, 
  AdjointSphere, AdjointMesh, AdjointMeshOpacity, AdjointSphereOpacity, 
  AdjointSphereRadius, AdjointArrowLength, 
  NonMeshAbsArgPlot3D, NonMeshArgDensityPlot, ContoursOpacity]

EndPackage[]
